@@ -10,8 +10,8 @@
 #![forbid(unsafe_code)]
 
 use std::{
-    collections::HashMap,
     fs::File,
+    os::fd::OwnedFd,
     sync::{Arc, Condvar, Mutex},
 };
 
@@ -21,8 +21,11 @@ use nix::{errno::Errno, unistd::Pid};
 use serde::{ser::SerializeMap, Serializer};
 
 use crate::{
-    elf::ExecutableFile, fs::CanonicalPath, hash::SydRandomState, ScmpNotifReq, SydArch,
-    SydMemoryMap, SydSigSet,
+    confine::{ScmpNotifReq, SydArch, SydMemoryMap},
+    elf::ExecutableFile,
+    hash::SydHashMap,
+    lookup::CanonicalPath,
+    sigset::SydSigSet,
 };
 
 /// Metadata on a blocking syscall invocation
@@ -32,22 +35,27 @@ pub struct SysInterrupt {
     pub handler: Pid,
     /// System call request
     pub request: ScmpNotifReq,
+    /// proc_pid_status(5) file handle
+    pub status: Option<OwnedFd>,
+    /// Used by syd_emu to signal syd_int to delete the entry and close the file.
+    /// This is because the status file descriptor is not valid in syd_emu's fs space.
+    pub delete: bool,
     /// True if `SA_RESTART` is ignored
     /// (e.g. due to a socket timeout).
     pub ignore_restart: bool,
 }
 
 /// Map of metadata on blocking syscall invocations.
-pub type BlockMap = HashMap<u64, SysInterrupt, SydRandomState>;
+pub type BlockVec = Vec<SysInterrupt>;
 
 /// Map of restarting signals by TGID.
-pub type RestartMap = HashMap<Pid, SydSigSet, SydRandomState>;
+pub type RestartMap = SydHashMap<Pid, SydSigSet>;
 
 /// This is the data type used to handle syscall interrupts.
 #[derive(Debug)]
 pub struct SysInterruptMap {
     /// Map of blocking syscalls by request id.
-    pub sys_block: Arc<(Mutex<BlockMap>, Condvar)>,
+    pub sys_block: Arc<(Mutex<BlockVec>, Condvar)>,
     /// Map of restarting signals by TGID.
     /// Used for SA_RESTART tracking.
     pub sig_restart: Arc<Mutex<RestartMap>>,
@@ -58,7 +66,6 @@ pub struct SysInterruptMap {
 pub struct ExecResult {
     pub(crate) exe: ExecutableFile,
     pub(crate) file: File,
-    pub(crate) arch: u32,
     pub(crate) ip: u64,
     pub(crate) sp: u64,
     pub(crate) args: [u64; 6],
@@ -67,32 +74,17 @@ pub struct ExecResult {
     pub(crate) memmap: Option<Vec<SydMemoryMap>>,
 }
 
-/// Represents a sigreturn(2) check result
-#[derive(Debug)]
-pub struct SigreturnResult {
-    pub(crate) is_realtime: bool,
-    pub(crate) ip: u64,
-    pub(crate) sp: u64,
-    pub(crate) args: [u64; 6],
-    pub(crate) ip_mem: Option<[u8; 64]>,
-    pub(crate) sp_mem: Option<[u8; 64]>,
-}
-
 /// Syscall-agnostic error map.
-pub type ErrorMap = HashMap<Pid, Option<Errno>, SydRandomState>;
+pub type ErrorMap = SydHashMap<Pid, Option<Errno>>;
 
 /// chdir(2) result map.
-pub type ChdirMap<'a> = HashMap<Pid, CanonicalPath<'a>, SydRandomState>;
+pub type ChdirMap<'a> = SydHashMap<Pid, CanonicalPath<'a>>;
 
 /// exec(3) result map.
-pub type ExecvMap = HashMap<Pid, ExecResult, SydRandomState>;
-
-/// {rt_,}sigreturn(2) result map.
-pub type SigretMap = HashMap<Pid, SigreturnResult, SydRandomState>;
+pub type ExecvMap = SydHashMap<Pid, ExecResult>;
 
 /// Results map for ptrace(2) hooks chdir, execve, sigaction and sigreturn.
 #[derive(Debug)]
-#[allow(clippy::type_complexity)]
 pub struct SysResultMap<'a> {
     /// syscall-agnostic error map
     pub trace_error: Arc<Mutex<ErrorMap>>,
@@ -100,18 +92,15 @@ pub struct SysResultMap<'a> {
     pub trace_chdir: Arc<Mutex<ChdirMap<'a>>>,
     /// exec(3) result map
     pub trace_execv: Arc<Mutex<ExecvMap>>,
-    /// {rt_,}sigreturn(2) result map
-    pub trace_sigret: Arc<Mutex<SigretMap>>,
 }
 
 /// Map of TGIDs that have received count signals for handled signals.
-pub type SighandleMap = HashMap<Pid, u64, SydRandomState>;
+pub type SighandleMap = SydHashMap<Pid, u64>;
 
 /// Signal map, used by signal counting for SROP mitigation:
 /// If a TGID is not in sig_handle_map at the entry of sigreturn(2),
 /// we terminate the process because the sigreturn(2) is artificial.
 #[derive(Debug)]
-#[allow(clippy::type_complexity)]
 pub struct SignalMap {
     /// Set of TGIDs that have received count signals for handled signals.
     pub sig_handle: Arc<Mutex<SighandleMap>>,
@@ -127,8 +116,27 @@ impl SysInterrupt {
             handler,
             request,
             ignore_restart,
+            status: None,
+            delete: false,
         })
     }
+
+    // Marks the interrupt for deletion as needed.
+    //
+    // Returns true if drop should be handled by syd_int.
+    pub(crate) fn delete(&mut self) -> bool {
+        // interrupt.status is Some if syd_int thread
+        // has already opened proc_pid_status(5), in
+        // which case we let it close the file because
+        // the file descriptor is not valid in syd_emu's
+        // FS space.
+        if self.status.is_some() {
+            self.delete = true;
+            true // syd_int drops interrupt.
+        } else {
+            false // syd_emu drops interrupt.
+        }
+    }
 }
 
 impl serde::Serialize for SysInterrupt {
@@ -155,7 +163,7 @@ impl serde::Serialize for SysInterrupt {
 /// Create a new SysInterruptMap.
 pub fn sys_interrupt_map_new() -> SysInterruptMap {
     SysInterruptMap {
-        sys_block: Arc::new((Mutex::new(BlockMap::new()), Condvar::new())),
+        sys_block: Arc::new((Mutex::new(BlockVec::new()), Condvar::new())),
         sig_restart: Arc::new(Mutex::new(RestartMap::new())),
     }
 }
@@ -166,7 +174,6 @@ pub fn sys_result_map_new<'a>() -> SysResultMap<'a> {
         trace_error: Arc::new(Mutex::new(ErrorMap::new())),
         trace_chdir: Arc::new(Mutex::new(ChdirMap::new())),
         trace_execv: Arc::new(Mutex::new(ExecvMap::new())),
-        trace_sigret: Arc::new(Mutex::new(SigretMap::new())),
     }
 }
 
