@@ -35,7 +35,7 @@ use nix::{
 };
 use serde::{ser::SerializeMap, Serialize, Serializer};
 
-use crate::{config::*, fs::FileType, XPath};
+use crate::{config::*, lookup::FileType, XPath};
 
 // x32 compatibility
 // See https://sourceware.org/bugzilla/show_bug.cgi?id=16437
