@@ -44,13 +44,14 @@ use crate::{
     config::{FADVISE_SYSCALLS, HAVE_RWF_NOAPPEND, MMAP_MIN_ADDR, SYD_MADVISE, UNSAFE_PERSONAS},
     cookie::safe_socket,
     err::{err2no, SydResult},
-    fs::{nlmsg_align, readlinkat, safe_clone, FileType, AT_BADFD},
+    fs::{nlmsg_align, readlinkat, safe_clone, AT_BADFD},
     info,
     landlock::{
         path_beneath_rules, Access, AccessFs, AccessNet, NetPort, RestrictSelfFlags,
         RestrictionStatus, Ruleset, RulesetAttr, RulesetCreatedAttr, RulesetError, RulesetStatus,
         Scope, ABI,
     },
+    lookup::FileType,
     path::{mask_path, XPathBuf},
     retry::retry_on_eintr,
     sandbox::{IoctlMap, Sandbox},
