@@ -42,7 +42,7 @@ pub fn resolve_rand(name: &str, family: Option<i32>) -> Result<IpAddr, Errno> {
 
     // Select a random IP address from the list.
     // Calculate random index within the bounds of the addresses vector.
-    #[allow(clippy::arithmetic_side_effects)]
+    #[expect(clippy::arithmetic_side_effects)]
     Ok(addrs[cookie.wrapping_rem(addrs.len())])
 }
 
@@ -79,7 +79,7 @@ pub fn resolve_host(name: &str, family: Option<i32>) -> Result<Vec<IpAddr>, Errn
 }
 
 /// Performs a reverse DNS lookup for the given IP address, returning a hostname or an error.
-#[allow(clippy::cast_possible_truncation)]
+#[expect(clippy::cast_possible_truncation)]
 pub fn lookup_addr(addr: IpAddr) -> Result<String, Errno> {
     let addr = match addr {
         IpAddr::V4(v4) => SockaddrStorage::from(SocketAddrV4::new(v4, 0)),
