@@ -1028,7 +1028,7 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
         let tmp = unsafe { BorrowedFd::borrow_raw(tmp) };
         mkstempat(tmp, b"syd-aes-")
     } else {
-        safe_memfd_create(c"syd-aes", *SAFE_MFD_FLAGS)
+        safe_memfd_create(c"syd/aes", *SAFE_MFD_FLAGS)
     }?;
 
     let iv = if let Some((iv, hmac_tag)) = iv_and_tag {
