@@ -21,8 +21,8 @@ use nix::{
 
 use crate::{
     config::PROC_FD,
-    fs::FsFlags,
     kernel::{syscall_path_handler, to_atflags, to_mode},
+    lookup::FsFlags,
     path::XPathBuf,
     req::{PathArgs, SysArg, UNotifyEventRequest},
     sandbox::SandboxGuard,
