@@ -17,8 +17,8 @@ use nix::{
 use crate::{
     config::PROC_FILE,
     confine::scmp_arch_bits,
-    fs::{file_type, FsFlags},
     kernel::syscall_path_handler,
+    lookup::{file_type, FsFlags},
     path::XPathBuf,
     req::{SysArg, UNotifyEventRequest},
 };
