@@ -13,8 +13,9 @@ use nix::{errno::Errno, NixPath};
 
 use crate::{
     compat::AddWatchFlags,
-    fs::{file_type, inotify_add_watch, FsFlags},
+    fs::inotify_add_watch,
     kernel::syscall_path_handler,
+    lookup::{file_type, FsFlags},
     path::XPathBuf,
     req::{SysArg, UNotifyEventRequest},
 };
