@@ -23,8 +23,8 @@ use crate::{
     compat::{fstatx, STATX_MODE},
     config::PROC_FILE,
     cookie::safe_linkat,
-    fs::{FileType, FsFlags},
     kernel::{syscall_path_handler, to_atflags},
+    lookup::{FileType, FsFlags},
     path::{XPath, XPathBuf},
     req::{PathArgs, SysArg, SysFlags, UNotifyEventRequest},
 };
