@@ -24,8 +24,9 @@ use crate::{
     config::{PAGE_SIZE, PROC_FILE},
     elf::ExecutableFile,
     error,
-    fs::{fd_status_flags, safe_open_msym, CanonicalPath},
+    fs::fd_status_flags,
     kernel::sandbox_path,
+    lookup::{safe_open_msym, CanonicalPath},
     path::XPathBuf,
     proc::{proc_mem, proc_stat, proc_statm},
     req::UNotifyEventRequest,
