@@ -15,8 +15,8 @@ use nix::{
 };
 
 use crate::{
-    fs::FsFlags,
     kernel::{syscall_path_handler, to_mode, to_sflag},
+    lookup::FsFlags,
     proc::proc_umask,
     req::{PathArgs, SysArg, UNotifyEventRequest},
 };
