@@ -24,11 +24,11 @@ use nix::{
 
 use crate::{
     cookie::safe_bind,
-    fs::CanonicalPath,
-    hook::UNotifyEventRequest,
     info,
+    lookup::CanonicalPath,
     path::XPathBuf,
     proc::proc_umask,
+    req::UNotifyEventRequest,
     sandbox::{Action, AddressPattern, Capability, CidrRule},
     wildmatch::MatchMethod,
 };
@@ -149,9 +149,24 @@ pub(crate) fn handle_bind(
                     }
                     _ => {
                         // Case 3: unnamed UNIX socket.
-                        //
-                        // SAFETY: Use dummy path `!unnamed' for unnamed UNIX sockets.
-                        let unix = XPathBuf::from("!unnamed");
+                        let unix = if addr.len() as usize == size_of::<libc::sa_family_t>() {
+                            // Autobind on abstract UNIX socket.
+                            getsockname::<SockaddrStorage>(fd.as_raw_fd())?
+                                .as_unix_addr()
+                                .ok_or(Errno::EINVAL)?
+                                .as_abstract()
+                                .map(|path| {
+                                    // SAFETY: Prefix UNIX abstract sockets with `@' before access check.
+                                    let mut unix = XPathBuf::from("@");
+                                    let null = memchr::memchr(0, path).unwrap_or(path.len());
+                                    unix.append_bytes(&path[..null]);
+                                    unix
+                                })
+                                .ok_or(Errno::EINVAL)?
+                        } else {
+                            // Use dummy path `!unnamed' for unnamed UNIX sockets.
+                            XPathBuf::from("!unnamed")
+                        };
 
                         // Handle bind_map after successful bind for UNIX sockets.
                         // We ignore errors because there's nothing we can do
@@ -168,6 +183,9 @@ pub(crate) fn handle_bind(
                     }
                 };
 
+                info!("ctx": "bind", "op": "allow_safe_bind",
+                    "sys": "bind", "pid": request.scmpreq.pid().as_raw(), "unix": &unix,
+                    "msg": format!("add rule `allow/net/connect+{unix}' after bind"));
                 let mut sandbox = request.get_mut_sandbox();
                 let acl = sandbox.get_acl_mut(Capability::CAP_NET_CONNECT);
                 if let Some(idx) = acl.iter().position(|(p, m, a)| {
