@@ -30,7 +30,7 @@ use crate::{
     },
     config::HOOK_SCKCALLS,
     confine::{op2errno, op2name, scmp_arch_bits},
-    fs::{fd_status_flags, file_type, safe_canonicalize, CanonicalPath, FileType, FsFlags},
+    fs::fd_status_flags,
     kernel::net::{
         accept::handle_accept,
         bind::handle_bind,
@@ -44,6 +44,7 @@ use crate::{
         sendto::handle_sendto,
         socket::{handle_socket, handle_socketpair},
     },
+    lookup::{file_type, safe_canonicalize, CanonicalPath, FileType, FsFlags},
     path::{XPath, XPathBuf},
     req::UNotifyEventRequest,
     sandbox::{Action, Capability, SandboxGuard},
