@@ -18,7 +18,7 @@ use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
     sys::{
-        socket::{MsgFlags, SockaddrLike, SockaddrStorage},
+        socket::{SockaddrLike, SockaddrStorage},
         time::TimeSpec,
     },
 };
@@ -26,15 +26,15 @@ use zeroize::Zeroizing;
 
 use crate::{
     compat::{
-        cmsg_len_32, cmsg_space_32, cmsghdr, cmsghdr32, iovec32, mmsghdr, mmsghdr32, msghdr,
-        msghdr32,
+        cmsg_len_32, cmsg_space_32, cmsghdr, cmsghdr32, iovec32, mmsghdr, mmsghdr32, mmsghdr_union,
+        msghdr, msghdr32, MsgFlags,
     },
     config::MMAP_MIN_ADDR,
     confine::scmp_arch_bits,
     fs::{get_nonblock, has_recv_timeout, peer_inode},
-    hook::UNotifyEventRequest,
     kernel::net::to_msgflags,
     path::XPath,
+    req::UNotifyEventRequest,
     sandbox::Flags,
 };
 
@@ -126,9 +126,6 @@ pub(crate) fn handle_recvmsg(
     #[expect(clippy::cast_sign_loss)]
     let r_bytes = result? as usize;
 
-    // Replace msg_iov pointer with the sandbox process pointer.
-    hdr.msg_iov = user_iov_base as *mut libc::iovec;
-
     // Scatter payload back into the sandbox process.
     if r_bytes > 0 {
         let mut remaining = r_bytes;
@@ -143,60 +140,54 @@ pub(crate) fn handle_recvmsg(
     }
 
     // Handle peer address logic.
-    if !hdr.msg_name.is_null() && hdr.msg_namelen > 0 {
-        let r_addr = if !hdr.msg_name.is_null() && hdr.msg_namelen > 0 {
-            // SAFETY: `hdr` is returned by the host kernel.
-            unsafe {
-                SockaddrStorage::from_raw(
-                    hdr.msg_name as *const libc::sockaddr,
-                    Some(hdr.msg_namelen),
-                )
-            }
+    let r_addr = if !hdr.msg_name.is_null() && hdr.msg_namelen > 0 {
+        // SAFETY: `hdr` is returned by the host kernel.
+        unsafe {
+            SockaddrStorage::from_raw(hdr.msg_name as *const libc::sockaddr, Some(hdr.msg_namelen))
+        }
+    } else {
+        None
+    };
+
+    // Modify source address if needed.
+    if let Some(mut addr) = r_addr {
+        let hdr_namelen = if let Some(peer_addr) = addr
+            .as_unix_addr()
+            .and_then(|u| u.path())
+            .map(|p| XPath::from_bytes(p.as_os_str().as_bytes()))
+            .filter(|p| p.starts_with(b"./"))
+            .map(|p| p.split().1)
+            .and_then(|base| request.find_unix_addr(base).ok())
+            .and_then(|sa| {
+                // SAFETY: `sa` originates from our bookkeeping; valid UnixAddr.
+                unsafe { SockaddrStorage::from_raw(sa.as_ptr().cast(), Some(sa.len())) }
+            }) {
+            addr = peer_addr;
+            addr.len()
         } else {
-            None
+            hdr.msg_namelen
         };
 
-        // Modify source address if needed.
-        if let Some(mut addr) = r_addr {
-            let hdr_namelen = if let Some(peer_addr) = addr
-                .as_unix_addr()
-                .and_then(|u| u.path())
-                .map(|p| XPath::from_bytes(p.as_os_str().as_bytes()))
-                .filter(|p| p.starts_with(b"./"))
-                .map(|p| p.split().1)
-                .and_then(|base| request.find_unix_addr(base).ok())
-                .and_then(|sa| {
-                    // SAFETY: `sa` originates from our bookkeeping; valid UnixAddr.
-                    unsafe { SockaddrStorage::from_raw(sa.as_ptr().cast(), Some(sa.len())) }
-                }) {
-                addr = peer_addr;
-                addr.len()
-            } else {
-                hdr.msg_namelen
-            };
-
-            // Write back truncated address.
-            // SAFETY: Convert SockaddrStorage to byte slice late, then truncate.
-            let bytes = unsafe {
-                std::slice::from_raw_parts(addr.as_ptr().cast::<u8>(), addr.len() as usize)
-            };
-            #[expect(clippy::cast_possible_truncation)]
-            let out_len = addr
-                .len()
-                .min(hdr_namelen)
-                .min(user_nam_size as libc::socklen_t);
-            // This write may fail if `user_nam_base` is not writable.
-            // Therefore, we should handle EFAULT gracefully.
-            // `process_msghdr_name` has already validated that
-            // this pointer is not below mmap_min_addr.
-            match request.write_mem(&bytes[..out_len as usize], user_nam_base) {
-                Ok(_) | Err(Errno::EFAULT) => {}
-                Err(errno) => return Err(errno),
-            }
-            hdr.msg_namelen = out_len;
-        } else {
-            hdr.msg_namelen = 0;
+        // Write back truncated address.
+        // SAFETY: Convert SockaddrStorage to byte slice late, then truncate.
+        let bytes =
+            unsafe { std::slice::from_raw_parts(addr.as_ptr().cast::<u8>(), addr.len() as usize) };
+        #[expect(clippy::cast_possible_truncation)]
+        let out_len = addr
+            .len()
+            .min(hdr_namelen)
+            .min(user_nam_size as libc::socklen_t);
+        // This write may fail if `user_nam_base` is not writable.
+        // Therefore, we should handle EFAULT gracefully.
+        // `process_msghdr_name` has already validated that
+        // this pointer is not below mmap_min_addr.
+        match request.write_mem(&bytes[..out_len as usize], user_nam_base) {
+            Ok(_) | Err(Errno::EFAULT) => {}
+            Err(errno) => return Err(errno),
         }
+        hdr.msg_namelen = out_len;
+    } else {
+        hdr.msg_namelen = 0;
     }
 
     // Handle SCM_RIGHTS and SCM_CREDENTIALS in the control message for each result.
@@ -323,12 +314,16 @@ pub(crate) fn handle_recvmsg(
             request.write_mem(&out_cmsg, user_ctl_base)?;
         }
 
-        hdr.msg_control = user_ctl_base as *mut libc::c_void;
         hdr.msg_controllen = cmsg_len;
     } else {
         hdr.msg_controllen = 0;
     }
 
+    // Replace local pointers with sandbox process pointers.
+    hdr.msg_iov = user_iov_base as *mut libc::iovec;
+    hdr.msg_name = user_nam_base as *mut libc::c_void;
+    hdr.msg_control = user_ctl_base as *mut libc::c_void;
+
     // Write-back msghdr handling 32-bit as necessary.
     if is32 {
         let m32: msghdr32 = hdr.into();
@@ -751,18 +746,21 @@ fn handle_recvmmsg_internal(
         }
 
         // Write back mmsghdr for each result in the array.
-        let m32: mmsghdr32;
-        let m64: mmsghdr;
-        let msg_header = if is32 {
-            m32 = mmsghdr32::from(*mmsg_hdr);
-            let ptr = &raw const m32 as *const u8;
-            // SAFETY: Writing directly from the raw memory of mmsghdr32.
-            unsafe { std::slice::from_raw_parts(ptr, size_of::<mmsghdr32>()) }
-        } else {
-            m64 = (*mmsg_hdr).into();
-            let ptr = &raw const m64 as *const u8;
-            // SAFETY: Writing directly from the raw memory of mmsghdr.
-            unsafe { std::slice::from_raw_parts(ptr, size_of::<mmsghdr>()) }
+        let m: mmsghdr_union;
+        // SAFETY:
+        // Writing directly from the raw memory of the union `m`,
+        // union is initialized to the correct element before write,
+        // and it remains alive as long as the `msg_header` pointer.
+        let msg_header = unsafe {
+            if is32 {
+                m.m32 = mmsghdr32::from(*mmsg_hdr);
+                let ptr = &raw const m.m32 as *const u8;
+                std::slice::from_raw_parts(ptr, size_of::<mmsghdr32>())
+            } else {
+                m.m64 = (*mmsg_hdr).into();
+                let ptr = &raw const m.m64 as *const u8;
+                std::slice::from_raw_parts(ptr, size_of::<mmsghdr>())
+            }
         };
 
         let msg_header_size = msg_header.len() as u64;
@@ -1091,9 +1089,10 @@ fn process_msghdr_name(hdr: &mut msghdr, nam_buf: &mut Vec<u8>) -> Result<(u64,
         *nam_buf = name;
         Ok((base, siz))
     } else {
+        let base = hdr.msg_name as u64;
         hdr.msg_name = std::ptr::null_mut();
         hdr.msg_namelen = 0;
-        Ok((0, 0))
+        Ok((base, 0))
     }
 }
 
@@ -1128,9 +1127,10 @@ fn process_msghdr_ctl(
         *ctl_buf = ctl;
         Ok((base, size))
     } else {
+        let base = hdr.msg_control as u64;
         hdr.msg_control = std::ptr::null_mut();
         hdr.msg_controllen = 0;
-        Ok((0, 0))
+        Ok((base, 0))
     }
 }
 
