@@ -182,7 +182,7 @@ pub(crate) fn handle_sendmsg(
         for fds in control_fds {
             // SAFETY: Check safe file types for SCM_RIGHTS.
             for fd in fds {
-                request.check_scm_rights(&fd, flags, 0x10, verbose)?;
+                request.check_scm_rights(fd, flags, 0x10, verbose)?;
             }
 
             // SAFETY: OwnedFd is repr(transparent) over RawFd, so
@@ -544,7 +544,7 @@ pub(crate) fn handle_sendmmsg(
                 for fds in control_fds {
                     // SAFETY: Check safe file types for SCM_RIGHTS.
                     for fd in fds {
-                        request.check_scm_rights(&fd, flags, 0x14, verbose)?;
+                        request.check_scm_rights(fd, flags, 0x14, verbose)?;
                     }
 
                     // SAFETY: OwnedFd is repr(transparent) over RawFd, so
