@@ -37,12 +37,12 @@ use crate::{
     cookie::{safe_memfd_create, safe_openat2, safe_uname},
     err::err2no,
     error,
-    fs::{
-        format_oflags, is_dev_kfd, oflag_accmode, oflag_nonblock, safe_open_how_msym,
-        safe_open_msym, seal_memfd, CanonicalPath, FileInfo, FileType, FsFlags, MaybeFd,
-    },
+    fs::{format_oflags, is_dev_kfd, oflag_accmode, oflag_nonblock, seal_memfd},
     hash::aes_ctr_tmp,
     info, log_enabled,
+    lookup::{
+        safe_open_how_msym, safe_open_msym, CanonicalPath, FileInfo, FileType, FsFlags, MaybeFd,
+    },
     mask::mask_proc_pid_status,
     ofd::{lock_fd, unlock_fd},
     path::{XPath, XPathBuf},
@@ -199,6 +199,9 @@ fn syscall_open_handler(
         //   2. flags is mutable because trace/allow_unsafe_open:0
         //      may change its value for O_PATH. Append-only may
         //      also edit flags.
+        //   3. flags is mutable because trace/allow_unsafe_create:0
+        //      may insert O_NOFOLLOW when O_CREAT is set but O_EXCL
+        //      is not set.
         let mut o_cloexec = flags.contains(OFlag::O_CLOEXEC);
         let mut flags = flags | OFlag::O_CLOEXEC;
 
@@ -280,6 +283,13 @@ fn syscall_open_handler(
             arg.fsflags.insert(FsFlags::NO_RESOLVE_DOTDOT);
         }
 
+        // SAFETY: Prevent file creation through dangling symlinks.
+        let restrict_create = !sandbox.flags.allow_unsafe_create();
+        if o_creat && restrict_create && !flags.contains(OFlag::O_EXCL) {
+            flags.insert(OFlag::O_NOFOLLOW);
+            arg.fsflags.insert(FsFlags::NO_FOLLOW_LAST);
+        }
+
         // Read the remote path and check for magic path.
         let (mut path, magic, _) = request.read_path(&sandbox, arg, true)?;
 
@@ -310,7 +320,7 @@ fn syscall_open_handler(
             }
 
             let fd = if caps.contains(Capability::CAP_READ) {
-                let mut file = safe_memfd_create(c"syd-box", *SAFE_MFD_FLAGS).map(File::from)?;
+                let mut file = safe_memfd_create(c"syd/box", *SAFE_MFD_FLAGS).map(File::from)?;
 
                 const FLEN: usize = MAGIC_PREFIX.len() + 3; /* .{el,sh} */
                 let fack = path.abs().len() == FLEN;
@@ -453,20 +463,25 @@ fn syscall_open_handler(
 
         // SAFETY:
         // 1. We must provide safe access to sandbox process' controlling terminal.
-        // 2. Both "/dev/tty" and the original tty path are checked for access.
+        // 2. Both the virtual "/dev/console" or "/dev/tty" and the replacing tty
+        //    path are checked for access.
         let pid = req.pid();
-        let dev_tty = Cow::Borrowed(XPath::from_bytes(b"/dev/tty"));
-        let has_tty = if path.abs().is_equal(dev_tty.as_bytes()) {
-            let dev_tty = proc_tty(pid)?;
-            if !request.is_valid() {
-                return Err(Errno::ESRCH);
+        let mut path_tty = None;
+        if path.is_char_device() {
+            const TTY: &[&[u8]] = &[b"/dev/tty", b"/dev/console"];
+            for p in TTY {
+                if path.abs().is_equal(p) {
+                    let real_tty = proc_tty(pid)?;
+                    if !request.is_valid() {
+                        return Err(Errno::ESRCH);
+                    }
+                    path = CanonicalPath::new_tty(real_tty)?;
+                    path_tty = Some(Cow::Borrowed(XPath::from_bytes(p)));
+                    break;
+                }
             }
-            path = CanonicalPath::new_tty(dev_tty)?;
-
-            true // TTY.
-        } else {
-            false // Not a TTY.
-        };
+        }
+        let has_tty = path_tty.is_some();
 
         // SAFETY: Ensure the sandbox process cannot
         // acquire a new controlling terminal other
@@ -502,28 +517,31 @@ fn syscall_open_handler(
                 Cow::Borrowed(path.abs())
             };
 
-            if !has_tty {
+            // If open is a TTY both the virtual and real path are checked for access.
+            // The more critical action wins.
+            let mut flip_tty = false;
+            for (idx, path) in [Some(&path_check), path_tty.as_ref()]
+                .iter()
+                .flatten()
+                .enumerate()
+            {
                 for cap in caps {
-                    let (new_action, new_filter) = sandbox.check_path(cap, &path_check);
+                    let (new_action, new_filter) = sandbox.check_path(cap, path);
                     if new_action >= action {
                         action = new_action;
                         filter = new_filter;
-                    }
-                }
-            } else {
-                // Both "/dev/tty" and the original tty path are checked for access.
-                // The more critical action wins.
-                for path in [&path_check, &dev_tty] {
-                    for cap in caps {
-                        let (new_action, new_filter) = sandbox.check_path(cap, path);
-                        if new_action >= action {
-                            action = new_action;
-                            filter = new_filter;
+                        if idx > 0 && new_action > Action::Allow {
+                            flip_tty = true;
                         }
                     }
                 }
             }
-            path_check
+            #[expect(clippy::disallowed_methods)]
+            if flip_tty {
+                path_tty.unwrap()
+            } else {
+                path_check
+            }
         } else {
             Cow::Borrowed(path.abs())
         };
@@ -595,7 +613,6 @@ fn syscall_open_handler(
         let verbose = sandbox.verbose;
         let unsafe_open_kfd = sandbox.flags.allow_unsafe_open_kfd();
         let unsafe_open_path = sandbox.flags.allow_unsafe_open_path();
-        let restrict_create = !sandbox.flags.allow_unsafe_create();
         let filter_machine_id = !sandbox.flags.allow_unsafe_machine_id();
         let filter_proc_status = !sandbox.flags.allow_unsafe_proc_pid_status();
         let filter_uname = !sandbox.flags.allow_unsafe_uname();
@@ -669,23 +686,20 @@ fn syscall_open_handler(
             return Err(Errno::ENOTDIR);
         }
 
-        // SAFETY: Return EPERM for {/dev,/proc}/kmsg,
-        // so dmesg(1) falls back to syslog(2) which we provide.
-        // EPERM is fine as we do this after the access check
-        // so path hiding was already done as necessary.
-        if !has_tty
-            && ((path.typ == Some(FileType::Chr) && path.abs().is_equal(b"/dev/kmsg"))
-                || path.abs().is_equal(b"/proc/kmsg"))
-        {
+        // SAFETY: Return EPERM for {/dev,/proc}/kmsg, so dmesg(1) falls back to
+        // syslog(2) which we provide. EPERM is fine as we do this after the
+        // access check so path hiding was already done as necessary.
+        if !has_tty && path.is_char_device() && path.abs().is_equal(b"/dev/kmsg") {
+            return Err(Errno::EPERM);
+        }
+        if !has_tty && path.is_file() && path.abs().is_equal(b"/proc/kmsg") {
             return Err(Errno::EPERM);
         }
 
         // SAFETY: Do not follow (magic) symlinks after canonicalization.
         // Exception: Last component is allowed with O_NOFOLLOW.
-        if let Some(FileType::Lnk) = &path.typ {
-            if !flags.contains(OFlag::O_NOFOLLOW) {
-                return Err(Errno::ELOOP);
-            }
+        if path.is_symlink() && !flags.contains(OFlag::O_NOFOLLOW) {
+            return Err(Errno::ELOOP);
         }
 
         // SAFETY: Restrictions a la protected_{fifos,regular}
@@ -861,7 +875,7 @@ fn syscall_open_handler(
                         // Existing path: Construct path to proc(5) magic symlink,
                         // or pre-open file descriptor.
                         let fd = do_open(&request, &path, flags, is_blocking)?;
-                        do_open_post(fd, &path, filter_proc_status)?
+                        do_open_post(fd, path, filter_proc_status)?
                     }
                 } else if o_mask {
                     // Cannot be O_PATH or encrypted fd!
@@ -1231,12 +1245,12 @@ fn do_open_pre(
 
 fn do_open_post(
     mut fd: OwnedFd,
-    path: &CanonicalPath,
+    path: CanonicalPath,
     filter_proc_status: bool,
 ) -> Result<OwnedFd, Errno> {
     // SAFETY: Filter security-sensitive proc_pid_status(5) contents.
     if filter_proc_status && path.abs().is_proc_pid_status() {
-        fd = do_mask_proc_pid_status(fd)?;
+        fd = do_mask_proc_pid_status(fd, path)?;
     }
 
     Ok(fd)
@@ -1244,7 +1258,7 @@ fn do_open_post(
 
 fn do_open_machine_id() -> Result<OwnedFd, Errno> {
     // Open a memory fd to be sealed for write.
-    let mut mfd = do_open_memfd(c"syd-machine-id").map(BufWriter::new)?;
+    let mut mfd = do_open_memfd(c"syd/etc/machine-id").map(BufWriter::new)?;
 
     // Generate machine-id(5) contents.
     #[expect(clippy::disallowed_methods)]
@@ -1267,7 +1281,7 @@ fn do_open_machine_id() -> Result<OwnedFd, Errno> {
 
 fn do_open_proc_version(request: &UNotifyEventRequest) -> Result<OwnedFd, Errno> {
     // Open a memory fd to be sealed for write.
-    let mut mfd = do_open_memfd(c"syd-proc-version").map(BufWriter::new)?;
+    let mut mfd = do_open_memfd(c"syd/proc/version").map(BufWriter::new)?;
 
     // Read UtsName using uname(2) syscall.
     //
@@ -1295,7 +1309,7 @@ fn do_open_proc_version(request: &UNotifyEventRequest) -> Result<OwnedFd, Errno>
 
 fn do_open_proc_osrelease() -> Result<OwnedFd, Errno> {
     // Open a memory fd to be sealed for write.
-    let mut mfd = do_open_memfd(c"syd-proc-sys-kernel-osrelease").map(BufWriter::new)?;
+    let mut mfd = do_open_memfd(c"syd/proc/sys/kernel/osrelease").map(BufWriter::new)?;
 
     // Read static release string.
     let release = Sandbox::os_release();
@@ -1315,9 +1329,13 @@ fn do_open_proc_osrelease() -> Result<OwnedFd, Errno> {
     Ok(mfd.into())
 }
 
-fn do_mask_proc_pid_status(fd: OwnedFd) -> Result<OwnedFd, Errno> {
+fn do_mask_proc_pid_status(fd: OwnedFd, path: CanonicalPath) -> Result<OwnedFd, Errno> {
+    // Prepare the name of the memfd for identification.
+    let mut path = path.take();
+    path.replace_prefix(b"/", b"syd/")?;
+
     // Open a memory fd to be sealed for write.
-    let mut mfd = do_open_memfd(c"syd-proc-pid-status")?;
+    let mut mfd = path.with_nix_path(do_open_memfd)??;
 
     // Filter proc_pid_status(5) into the memory fd.
     mask_proc_pid_status(fd, &mfd)?;
