@@ -21,8 +21,9 @@ use crate::{
     elf::{ElfError, ElfFileType, ElfType, ExecutableFile, LinkingType},
     err::err2no,
     error,
-    fs::{is_executable, safe_open_msym, FileType, FsFlags, MaybeFd, AT_EXECVE_CHECK},
+    fs::{is_executable, AT_EXECVE_CHECK},
     kernel::{sandbox_path, to_atflags},
+    lookup::{safe_open_msym, FileType, FsFlags, MaybeFd},
     path::XPathBuf,
     ptrace::ptrace_syscall_info,
     req::{RemoteProcess, SysArg, SysFlags},
@@ -159,8 +160,9 @@ pub(crate) fn sysenter_exec(
 
         match action {
             Action::Allow | Action::Warn => {}
-            Action::Deny | Action::Filter => return Err(Errno::EACCES),
-            Action::Panic => panic!(),
+            Action::Panic | Action::Deny | Action::Filter => return Err(Errno::EACCES),
+            //Do NOT panic the main thread!
+            //Action::Panic => panic!(),
             Action::Exit => std::process::exit(libc::EACCES),
             Action::Stop => {
                 let _ = kill(process.pid, Some(Signal::SIGSTOP));
@@ -194,8 +196,9 @@ pub(crate) fn sysenter_exec(
         }
         match action {
             Action::Allow | Action::Warn => {}
-            Action::Deny | Action::Filter => return Err(Errno::EACCES),
-            Action::Panic => panic!(),
+            Action::Panic | Action::Deny | Action::Filter => return Err(Errno::EACCES),
+            //Do NOT panic the main thread!
+            //Action::Panic => panic!(),
             Action::Exit => std::process::exit(libc::EACCES),
             Action::Stop => {
                 let _ = kill(pid, Some(Signal::SIGSTOP));
@@ -390,12 +393,14 @@ pub(crate) fn sysenter_exec(
                     "tip": format!("configure `force+{path}:<checksum>'"));
             }
             Ok(Action::Filter) => return Err(Errno::EACCES),
-            Ok(Action::Deny) => {
-                warn!("ctx": "verify_elf", "act": Action::Deny,
+            Ok(act @ (Action::Deny | Action::Panic)) => {
+                warn!("ctx": "verify_elf", "act": act,
                     "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
                 return Err(Errno::EACCES);
             }
+            //Do NOT panic the main thread!
+            //Ok(Action::Panic) => panic!(),
             Ok(Action::Stop) => {
                 warn!("ctx": "verify_elf", "act": Action::Stop,
                     "pid": pid.as_raw(), "path": path.abs(),
@@ -423,7 +428,6 @@ pub(crate) fn sysenter_exec(
                     "tip": format!("configure `force+{path}:<checksum>'"));
                 std::process::exit(libc::EACCES);
             }
-            Ok(Action::Panic) => panic!(),
             Err(IntegrityError::Sys(errno)) => {
                 error!("ctx": "verify_elf",
                     "msg": format!("system error during ELF checksum calculation: {errno}"),
@@ -444,7 +448,9 @@ pub(crate) fn sysenter_exec(
                 }
                 match action {
                     Action::Allow | Action::Warn => {}
-                    Action::Filter | Action::Deny => return Err(Errno::EACCES),
+                    Action::Panic | Action::Deny | Action::Filter => return Err(Errno::EACCES),
+                    //Do NOT panic the main thread!
+                    //Action::Panic => panic!(),
                     Action::Stop | Action::Abort | Action::Kill => {
                         let _ = kill(
                             pid,
@@ -460,7 +466,6 @@ pub(crate) fn sysenter_exec(
                         );
                         return Err(Errno::EACCES);
                     }
-                    Action::Panic => panic!(),
                     Action::Exit => std::process::exit(libc::EACCES),
                 };
             }
