@@ -19,9 +19,10 @@ use crate::{
     compat::{fstatat64, statx, STATX_BASIC_STATS, STATX_MODE, STATX_TYPE},
     config::{MAGIC_PREFIX, MMAP_MIN_ADDR},
     confine::{scmp_arch_bits, EOWNERDEAD},
-    fs::{parse_fd, CanonicalPath, FileInfo, FileType, FsFlags},
+    fs::parse_fd,
     hash::SydHashSet,
     kernel::{sandbox_path, to_atflags},
+    lookup::{CanonicalPath, FileInfo, FileType, FsFlags},
     path::XPath,
     req::{SysArg, SysFlags, UNotifyEventRequest},
     sandbox::Capability,
@@ -333,13 +334,19 @@ fn syscall_stat_handler(
                 } // Lock is released here.
             }
 
+            // Return correct stat information for !memfd:syd/ paths.
+            // This prefix is internal to Syd and sandbox process cannot
+            // create memory file descriptors with this name prefix.
+            if is_fd && path.is_memory_fd() && path.abs().starts_with(b"!memfd:syd") {
+                let mut p = path.take();
+                p.drain(0..b"!memfd:syd".len());
+                path = CanonicalPath::new_mask(&p, &p)?;
+            }
+
             // SAFETY:
             // 1. Allow access to fd-only calls.
             // 2. Allow access to files with encryption in progress.
-            // 3. Allow access to !memfd:syd-*. This prefix is internal
-            //    to Syd and sandbox process cannot create memory file
-            //    descriptors with this name prefix.
-            if has_stat && arg.path.is_some() && !path.is_syd_memory_fd() {
+            if has_stat && arg.path.is_some() {
                 sandbox_path(
                     Some(&request),
                     &sandbox,
