@@ -9,17 +9,17 @@
 // SAFETY: This module has been liberated from unsafe code!
 #![forbid(unsafe_code)]
 
-use indexmap::IndexMap;
+use std::sync::LazyLock;
+
 use memchr::arch::all::is_equal;
 use nix::{
     errno::Errno,
     fcntl::{open, OFlag},
     sys::stat::Mode,
 };
-use once_cell::sync::Lazy;
 
 use crate::{
-    hash::{SydHashMap, SydHashSet},
+    hash::{SydHashMap, SydHashSet, SydIndexMap},
     landlock::{
         Access, AccessFs, AccessNet, CompatLevel, Compatible, CreateRulesetError, Errata, NetPort,
         PathBeneath, PathFd, RestrictSelfFlags, RestrictionStatus, Ruleset, RulesetAttr,
@@ -33,8 +33,8 @@ use crate::{
 };
 
 /// landlock(7) filesystem access rights.
-pub static LANDLOCK_ACCESS_FS: Lazy<IndexMap<&str, AccessFs>> = Lazy::new(|| {
-    IndexMap::from_iter([
+pub static LANDLOCK_ACCESS_FS: LazyLock<SydIndexMap<&str, AccessFs>> = LazyLock::new(|| {
+    SydIndexMap::from_iter([
         ("read", AccessFs::ReadFile),
         ("write", AccessFs::WriteFile),
         ("exec", AccessFs::Execute),
@@ -59,17 +59,20 @@ pub static LANDLOCK_ACCESS_FS: Lazy<IndexMap<&str, AccessFs>> = Lazy::new(|| {
         ("dpath", LandlockPolicy::access_fs_from_set("dpath")),
         ("spath", LandlockPolicy::access_fs_from_set("spath")),
         ("tpath", LandlockPolicy::access_fs_from_set("tpath")),
+        ("bnet", LandlockPolicy::access_fs_from_set("bnet")),
     ])
 });
 
 /// landlock(7) network access rights.
-pub static LANDLOCK_ACCESS_NET: Lazy<IndexMap<&str, AccessNet>> = Lazy::new(|| {
-    IndexMap::from_iter([
+pub static LANDLOCK_ACCESS_NET: LazyLock<SydIndexMap<&str, AccessNet>> = LazyLock::new(|| {
+    SydIndexMap::from_iter([
         ("bind", AccessNet::BindTcp),
         ("connect", AccessNet::ConnectTcp),
         // Aliases
         ("net", LandlockPolicy::access_net_from_set("net")),
         ("inet", LandlockPolicy::access_net_from_set("inet")),
+        ("bnet", LandlockPolicy::access_net_from_set("bnet")),
+        ("cnet", LandlockPolicy::access_net_from_set("cnet")),
     ])
 });
 
@@ -242,7 +245,7 @@ impl LandlockPolicy {
             std::mem::swap(&mut port0, &mut port1);
         }
         // FixedBitSet::insert_range does not support RangeInclusive.
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let ports = port0..(port1 + 1);
 
         for access in access.iter() {
@@ -271,7 +274,7 @@ impl LandlockPolicy {
             std::mem::swap(&mut port0, &mut port1);
         }
         // FixedBitSet::insert_range does not support RangeInclusive.
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let ports = port0..(port1 + 1);
 
         for access in access.iter() {
@@ -406,8 +409,10 @@ impl LandlockPolicy {
             AccessFs::MakeFifo | AccessFs::MakeSym
         } else if is_equal(s, b"tpath") {
             AccessFs::MakeDir | AccessFs::RemoveDir
+        } else if is_equal(s, b"bnet") {
+            AccessFs::MakeSock
         } else {
-            panic!("BUG: Invalid landlock(7) filesystem access right {set}");
+            unreachable!("BUG: Invalid landlock(7) filesystem access right {set}, report a bug!");
         }
     }
 
@@ -418,17 +423,19 @@ impl LandlockPolicy {
         let s = set.as_bytes();
         if is_equal(s, b"all") {
             AccessNet::all()
+        } else if is_equal(s, b"bnet") {
+            AccessNet::BindTcp
+        } else if is_equal(s, b"cnet") {
+            AccessNet::ConnectTcp
         } else if is_equal(s, b"net") || is_equal(s, b"inet") {
             AccessNet::BindTcp | AccessNet::ConnectTcp
         } else {
-            panic!("BUG: Invalid landlock(7) network access right {set}");
+            unreachable!("BUG: Invalid landlock(7) network access right {set}, report a bug!");
         }
     }
 
     /// A helper function to wrap the operations and reduce duplication.
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cognitive_complexity)]
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::cognitive_complexity)]
     pub fn restrict_self(&self, abi: ABI) -> Result<RestrictionStatus, RulesetError> {
         // from_all includes IoctlDev of ABI >= 5 as necessary.
         let mut ruleset = Ruleset::default().handle_access(AccessFs::from_all(abi))?;
@@ -693,7 +700,7 @@ impl LandlockPolicy {
             ruleset = ruleset.add_rules(landlock_path_beneath_rules(level, paths, *access))?;
         }
 
-        #[allow(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_possible_truncation)]
         ruleset
             .add_rules(network_rules_bind.ones().map(|port| {
                 Ok::<NetPort, RulesetError>(NetPort::new(port as u16, AccessNet::BindTcp))
@@ -738,8 +745,8 @@ impl LandlockPolicy {
 }
 
 // syd::landlock::path_beneath_rules tailored for Syd use-case.
-#[allow(clippy::cognitive_complexity)]
-#[allow(clippy::disallowed_methods)]
+#[expect(clippy::cognitive_complexity)]
+#[expect(clippy::disallowed_methods)]
 fn landlock_path_beneath_rules<I, P>(
     level: CompatLevel,
     paths: I,
@@ -757,7 +764,6 @@ where
 
     paths.into_iter().filter_map(move |p| {
         let p = p.as_ref();
-        #[allow(clippy::cast_possible_truncation)]
         match open(p, OFlag::O_PATH | OFlag::O_CLOEXEC, Mode::empty()) {
             Ok(fd) => Some(Ok(PathBeneath::new(PathFd { fd }, access))),
             Err(errno @ Errno::ENOENT) if level == CompatLevel::BestEffort => {
