@@ -33,6 +33,7 @@ use crate::{
     ofd::{lock_fd, unlock_fd},
     proc::{proc_cmdline, proc_comm, proc_cwd, proc_tty},
     retry::retry_on_eintr,
+    sandbox::Action,
     syslog::LogLevel,
 };
 
@@ -102,27 +103,24 @@ pub(crate) fn log_set_panic_hook() {
         let file = info.location().map(|l| l.file());
         let line = info.location().map(|l| l.line());
 
-        if log_is_main() {
+        if log_is_main(this.id()) {
             // Main thread panicking isn't recoverable unlike others.
             // Ensure clean exit right away.
-            crate::alert!("ctx": "panic",
-                "op": "panic_syd_main_thread",
-                "msg": err, "file": file, "line": line);
+            crate::alert!("ctx": "panic", "act": Action::Exit,
+                "name": "main", "msg": err, "file": file, "line": line);
             std::process::exit(101);
         } else {
-            crate::crit!("ctx": "panic",
-                "op": format!("panic_{name}_thread"),
-                "msg": err, "file": file, "line": line);
+            crate::crit!("ctx": "panic", "act": Action::Deny,
+                "name": name, "msg": err, "file": file, "line": line);
         }
     }));
 }
 
 // Use this to check for main thread in panic handler.
-#[inline]
-fn log_is_main() -> bool {
+pub(crate) fn log_is_main(tid: ThreadId) -> bool {
     LOG_MAIN_TID
         .get()
-        .map(|&tid| tid == std::thread::current().id())
+        .map(|&main_tid| main_tid == tid)
         .unwrap_or(false)
 }
 
@@ -398,13 +396,8 @@ pub fn log_init(default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resul
     init_global_syslog(logbuflen, fd, level, use_stack)?;
 
     // Finally let's make some noise!
-    //
-    // ENV_SING avoids duplicating this message over memfd-reexec.
-    if std::env::var_os(ENV_SING).is_none() {
-        info!("ctx": "init", "op": "sing", "chapter": 24,
-            "msg": "Change return success. Going and coming without error. Action brings good fortune.");
-        std::env::set_var(ENV_SING, "");
-    }
+    info!("ctx": "init", "op": "sing", "chapter": 24,
+        "msg": "Change return success. Going and coming without error. Action brings good fortune.");
 
     Ok(())
 }
@@ -631,13 +624,8 @@ pub fn log_init(_default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resu
     }
 
     // Finally let's make some noise!
-    //
-    // ENV_SING avoids duplicating this message over memfd-reexec.
-    if std::env::var_os(ENV_SING).is_none() {
-        info!("ctx": "init", "op": "sing", "chapter": 24,
-            "msg": "Change return success. Going and coming without error. Action brings good fortune.");
-        std::env::set_var(ENV_SING, "");
-    }
+    info!("ctx": "init", "op": "sing", "chapter": 24,
+        "msg": "Change return success. Going and coming without error. Action brings good fortune.");
 
     Ok(())
 }
