@@ -37,9 +37,7 @@ use nix::{
 };
 
 use crate::{
-    config::MAGIC_PREFIX,
-    fs::{tgkill, FileType},
-    log::log_untrusted_buf,
+    config::MAGIC_PREFIX, fs::tgkill, log::log_untrusted_buf, lookup::FileType,
     retry::retry_on_eintr,
 };
 
@@ -955,39 +953,23 @@ impl XPath {
 
     /// Checks if the path consists only of "." components.
     pub fn is_dot(&self) -> bool {
-        let bytes = self.as_bytes();
-        if bytes.is_empty() {
+        // Empty or absolute paths are not "."-only.
+        if self.is_empty() || self.is_absolute() {
             return false;
         }
 
-        let mut i = 0;
+        // Every non-empty component must be exactly ".".
         let mut saw_component = false;
-
-        #[expect(clippy::arithmetic_side_effects)]
-        while i < bytes.len() {
-            // Skip one or more '/'.
-            while i < bytes.len() && bytes[i] == b'/' {
-                i += 1;
-            }
-            if i == bytes.len() {
-                break; // trailing slashes only
-            }
-
-            // Start of a non-empty component.
+        for comp in self
+            .as_bytes()
+            .split(|&c| c == b'/')
+            .filter(|s| !s.is_empty())
+        {
             saw_component = true;
-
-            // Component must be exactly "."
-            if bytes[i] != b'.' {
-                return false;
-            }
-            i += 1;
-
-            // If the component continues, it must immediately be a '/' (or end).
-            if i < bytes.len() && bytes[i] != b'/' {
-                return false; // e.g., ".." or ".hidden"
+            if comp != b"." {
+                return false; // rejects "..", ".hidden", "a", etc.
             }
         }
-
         saw_component
     }
 
@@ -1625,8 +1607,8 @@ mod tests {
             ("././", true),
             ("./././", true),
             ("././././", true),
-            ("/././", true),
-            ("/./././", true),
+            ("/././", false),
+            ("/./././", false),
             (".//././", true),
             ("", false),
             ("/", false),
