@@ -240,21 +240,7 @@ impl ThreadPool {
             .name("syd_mon".to_string())
             .stack_size(MON_STACK_SIZE)
             .spawn(move || {
-                // Get information from the sandbox about:
-                // 1. Encryption if Crypt sandboxing is enabled.
-                // 2. Transit UIDs and GIDs for SafeSetId.
-                let (fpid, crypt, transit_uids, transit_gids) = {
-                    let sandbox = self.sandbox.read().unwrap_or_else(|err| err.into_inner());
-                    let fpid = sandbox.fpid;
-
-                    let crypt = if sandbox.enabled(Capability::CAP_CRYPT) {
-                        Some((sandbox.crypt_setup()?, sandbox.crypt_tmp))
-                    } else {
-                        None
-                    };
-
-                    (fpid, crypt, sandbox.transit_uids.clone(), sandbox.transit_gids.clone())
-                };
+                let sandbox = self.sandbox.read().unwrap_or_else(|err| err.into_inner());
 
                 // SAFETY: We use exit_group(2) here to bail,
                 // because this unsharing is a critical safety feature.
@@ -272,10 +258,16 @@ impl ThreadPool {
                     ROOT_FD() as libc::c_uint,
                     PROC_FD() as libc::c_uint,
                     NULL_FD() as libc::c_uint,
-                    fpid as libc::c_uint,
+                    sandbox.fpid as libc::c_uint,
                     self.seccomp_fd as libc::c_uint,
                     crate::log::LOG_FD.load(Ordering::Relaxed) as libc::c_uint,
                 ];
+
+                let crypt = if sandbox.enabled(Capability::CAP_CRYPT) {
+                    Some((sandbox.crypt_setup()?, sandbox.crypt_tmp))
+                } else {
+                    None
+                };
                 #[expect(clippy::cast_sign_loss)]
                 if let Some((crypt_fds, crypt_tmp)) = crypt {
                     set.push(crypt_fds.0 as libc::c_uint);
@@ -289,13 +281,19 @@ impl ThreadPool {
                 drop(set);
 
                 // Spawn the interrupt thread which will confine itself.
-                self.try_spawn_interrupt(&transit_uids, &transit_gids)?;
+                self.try_spawn_interrupt(&sandbox.transit_uids, &sandbox.transit_gids)?;
 
                 // Spawn the AES thread if encryption is on.
                 let crypt_handle = if let Some((fds, tmp)) = crypt {
                     let map = self.crypt_map.as_ref().map(Arc::clone).ok_or(Errno::ENOKEY)?;
                     let should_exit = Arc::clone(&self.should_exit);
-                    Some(self.try_spawn_aes(fds, map, tmp.is_none(), should_exit, &transit_uids, &transit_gids)?)
+                    Some(self.try_spawn_aes(
+                            fds,
+                            map,
+                            tmp.is_none(),
+                            should_exit,
+                            &sandbox.transit_uids,
+                            &sandbox.transit_gids)?)
                 } else {
                     None
                 };
@@ -319,7 +317,12 @@ impl ThreadPool {
                 let is_crypt = self.crypt_map.is_some();
 
                 let mut ctx = if !dry_run {
-                    let ctx = Worker::prepare_confine(self.seccomp_fd, self.flags, is_crypt, &transit_uids, &transit_gids)?;
+                    let ctx = Worker::prepare_confine(
+                        self.seccomp_fd,
+                        self.flags,
+                        is_crypt,
+                        &sandbox.transit_uids,
+                        &sandbox.transit_gids)?;
 
                     if Sandbox::locked_once() {
                         // Sandbox locked, confine right away.
@@ -363,8 +366,7 @@ impl ThreadPool {
                         "msg": "monitor thread is running unconfined in debug mode");
                     None
                 };
-                drop(transit_uids);
-                drop(transit_gids);
+                drop(sandbox); // release the read-lock.
 
                 info!("ctx": "boot", "op": "start_core_emulator_threads",
                     "msg": format!("starting {} core emulator thread{}, sandboxing started!",
