@@ -43,7 +43,7 @@ use btoi::{btoi, btoi_radix};
 use memchr::{arch::all::is_prefix, memrchr};
 use nix::{
     errno::Errno,
-    fcntl::{OFlag, ResolveFlag},
+    fcntl::{openat2, OFlag, ResolveFlag},
     libc::pid_t,
     sys::{
         stat::Mode,
@@ -68,8 +68,10 @@ use crate::{
     cookie::{PROCMAP_QUERY_COOKIE_ARG3, PROCMAP_QUERY_COOKIE_ARG4, PROCMAP_QUERY_COOKIE_ARG5},
     elf::ElfType,
     err::{err2no, proc_error_to_errno},
-    fs::{is_open_fd, parse_fd, read_all, readlinkat, safe_open, write_all, AT_BADFD},
+    fs::{is_empty_file, is_open_fd, parse_fd, readlinkat, AT_BADFD},
     hash::{SydHashMap, SydHashSet, SydIndexSet},
+    io::{read_all, write_all},
+    lookup::{safe_open, safe_open_how_abs},
     parsers::{
         map_result,
         proc::{
@@ -1991,6 +1993,26 @@ pub fn proc_kernel_taint() -> Result<KernelTaintFlags, Errno> {
     Ok(KernelTaintFlags::from_bits_retain(val))
 }
 
+/// Write the given $data to /proc/$pid/$name.
+pub fn proc_write(pid: Pid, name: &[u8], data: &[u8]) -> Result<(), Errno> {
+    let mut pfd = XPathBuf::from("/proc");
+    pfd.push_pid(pid);
+    pfd.push(name);
+
+    let how = safe_open_how_abs(OFlag::O_WRONLY, ResolveFlag::empty());
+
+    // This function runs early at startup before syd::config::proc_init.
+    #[expect(clippy::disallowed_methods)]
+    let fd = retry_on_eintr(|| openat2(AT_BADFD, &pfd, how))?;
+
+    // Check the file is an empty file or bail out.
+    if !is_empty_file(&fd).unwrap_or(false) {
+        return Err(Errno::EBADFD);
+    }
+
+    write_all(&fd, data)
+}
+
 #[cfg(test)]
 mod tests {
     use std::os::unix::fs::PermissionsExt;
