@@ -46,10 +46,10 @@ use crate::{
     confine::{op2name, scmp_arch_bits, ScmpNotifReq, SydArch, Sydcall, EIDRM},
     error,
     fs::{
-        fd_status_flags, file_type, pidfd_getfd, pidfd_open, pidfd_send_signal, process_mrelease,
-        safe_canonicalize, safe_open_msym, seccomp_notify_addfd, seccomp_notify_id_valid,
-        to_valid_fd, unix_inodes, CanonicalPath, FileType, FsFlags, PIDFD_THREAD,
+        fd_status_flags, pidfd_getfd, pidfd_open, pidfd_send_signal, process_mrelease,
+        seccomp_notify_addfd, seccomp_notify_id_valid, to_valid_fd, unix_inodes, PIDFD_THREAD,
     },
+    lookup::{file_type, safe_canonicalize, safe_open_msym, CanonicalPath, FileType, FsFlags},
     path::{XPath, XPathBuf, PATH_MAX, PATH_MIN},
     proc::{
         proc_auxv, proc_comm, proc_maps, proc_rand_fd, proc_stack_pointer, proc_status, proc_tgid,
