@@ -107,6 +107,14 @@ pub fn madvise_guard_remove(addr: NonNull<c_void>, len: NonZeroUsize) -> Result<
     Errno::result(unsafe { libc::madvise(addr.as_ptr(), len.get(), MADV_GUARD_REMOVE) }).map(drop)
 }
 
+// Get page size.
+fn getpagesize() -> Result<usize, Errno> {
+    sysconf(SysconfVar::PAGE_SIZE)?
+        .ok_or(Errno::EINVAL)?
+        .try_into()
+        .or(Err(Errno::EINVAL))
+}
+
 /// A box for a single `T` in a page-aligned, read/write anonymous mapping,
 /// which can be sealed to read-only. Supports uninitialized allocation,
 /// raw pointers, and zero-boilerplate conversion from `MaybeUninit`.
@@ -121,11 +129,8 @@ pub struct SealBox<T> {
 impl<T> SealBox<T> {
     /// Allocate and initialize `data` on a page-aligned, read/write anonymous mapping.
     pub fn new(data: T) -> Result<Self, Errno> {
-        #[expect(clippy::cast_possible_truncation)]
-        #[expect(clippy::cast_sign_loss)]
-        let page = sysconf(SysconfVar::PAGE_SIZE)?.ok_or(Errno::EINVAL)? as usize;
-
-        let size = mem::size_of::<T>();
+        let page = getpagesize()?;
+        let size = size_of::<T>();
         let (map_len, map_prot) = if size == 0 {
             // SAFETY: Zero-size, map a guard page.
             (page, ProtFlags::PROT_NONE)
@@ -211,11 +216,8 @@ impl<T> SealBox<T> {
 impl<T> SealBox<MaybeUninit<T>> {
     /// Allocate space for a `T` but do *not* initialize it.
     pub fn new_uninit() -> Result<Self, Errno> {
-        #[expect(clippy::cast_possible_truncation)]
-        #[expect(clippy::cast_sign_loss)]
-        let page = sysconf(SysconfVar::PAGE_SIZE)?.ok_or(Errno::EINVAL)? as usize;
-
-        let size = mem::size_of::<T>();
+        let page = getpagesize()?;
+        let size = size_of::<T>();
         let (map_len, map_prot) = if size == 0 {
             // SAFETY: Zero-size, map a guard page.
             (page, ProtFlags::PROT_NONE)
@@ -321,12 +323,8 @@ impl<E> SealBoxSlice<E> {
     where
         E: Clone,
     {
+        let page = getpagesize()?;
         let len = slice.len();
-
-        #[expect(clippy::cast_possible_truncation)]
-        #[expect(clippy::cast_sign_loss)]
-        let page = sysconf(SysconfVar::PAGE_SIZE)?.ok_or(Errno::EINVAL)? as usize;
-
         if len == 0 {
             // SAFETY: Zero-size, map a guard page.
             let map_len = NonZeroUsize::new(page).ok_or(Errno::EINVAL)?;
@@ -343,7 +341,7 @@ impl<E> SealBoxSlice<E> {
             });
         }
 
-        let elem_size = mem::size_of::<E>();
+        let elem_size = size_of::<E>();
         if elem_size == 0 {
             return Err(Errno::EINVAL);
         }
@@ -2244,7 +2242,7 @@ mod tests {
         }
         let page = sysconf(SysconfVar::PAGE_SIZE).unwrap().unwrap() as usize;
         // pick enough entries so that total data > 2 pages
-        let len = (page * 2 / mem::size_of::<String>()) + 5;
+        let len = (page * 2 / size_of::<String>()) + 5;
         let data = vec!["x".repeat(50); len];
         let sealed = SealBoxSlice::from_slice(&data).unwrap().seal(None).unwrap();
         assert_eq!(sealed.len(), len);
@@ -2253,7 +2251,7 @@ mod tests {
         }
         let (_ptr, map_len) = sealed.mapping();
         assert_eq!(map_len.get() % page, 0);
-        assert!(map_len.get() >= len * mem::size_of::<String>());
+        assert!(map_len.get() >= len * size_of::<String>());
     }
 
     #[test]
