@@ -54,8 +54,8 @@ use std::{
     env,
     env::VarError,
     ffi::{OsStr, OsString},
-    fs::{File, OpenOptions},
-    io::{BufWriter, Write},
+    fs::OpenOptions,
+    io::{stdout, BufWriter, Write},
     net::{IpAddr, SocketAddrV4, SocketAddrV6},
     os::{
         fd::AsRawFd,
@@ -68,45 +68,52 @@ use std::{
 
 use data_encoding::HEXLOWER;
 use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext, ScmpSyscall};
+use memchr::arch::all::is_equal;
 use nix::{
     errno::Errno,
-    fcntl::{open, OFlag},
+    fcntl::OFlag,
     libc::setdomainname,
-    pty::{grantpt, posix_openpt, ptsname_r, unlockpt},
+    pty::{grantpt, unlockpt},
     sched::{unshare, CloneFlags},
     sys::{
         socket::{bind, socket, AddressFamily, SockFlag, SockType, SockaddrIn, SockaddrIn6},
-        stat::Mode,
         wait::{Id, WaitPidFlag},
     },
     time::{clock_gettime, ClockId},
-    unistd::{fork, getgid, getpid, getuid, isatty, sethostname, ForkResult},
+    unistd::{fork, getgid, getpid, getuid, isatty, sethostname, ForkResult, Pid},
 };
 use sendfd::SendWithFd;
 use syd::{
     caps,
     compat::{waitid, WaitStatus},
     config::*,
-    confine::ExportMode,
+    confine::{confine_scmp_madvise, ExportMode},
     err::err2no,
     error,
-    fs::{closeexcept, format_clone_flags, format_clone_names, pidfd_open, set_cloexec},
-    hash::{get_at_random_hex, hash, HashAlgorithm, SydHashMap, SydHashSet},
+    fs::{
+        closeexcept, format_clone_flags, format_clone_names, openpt, openpts, pidfd_open,
+        set_cloexec,
+    },
+    hash::{get_at_random_hex, hash, HashAlgorithm, SydHashSet},
     hook::Supervisor,
     ignore_signals, info,
     landlock_policy::LandlockPolicy,
     log::log_init,
     loopback_set_up,
     path::XPathBuf,
-    printf,
+    proc::proc_write,
     sandbox::Sandbox,
     seal::ensure_sealed,
     set_sigpipe_dfl, syd_code_name, syd_info,
     syslog::LogLevel,
-    unshare::{GidMap, UidMap},
     warn, IgnoreSignalOpts,
 };
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
 // Set global allocator to tcmalloc if profiling is enabled.
 #[cfg(feature = "prof")]
 #[global_allocator]
@@ -154,12 +161,19 @@ syd::main! {
                     }
                     "--el" => {
                         set_sigpipe_dfl()?;
-                        printf!("{SYD_EL}")?;
+                        stdout().write_all(SYD_EL.as_bytes())?;
                         return Ok(ExitCode::SUCCESS);
                     }
                     "--sh" => {
                         set_sigpipe_dfl()?;
-                        printf!("{ESYD_SH}")?;
+                        stdout().write_all(ESYD_SH.as_bytes())?;
+                        return Ok(ExitCode::SUCCESS);
+                    }
+                    "--api" => {
+                        set_sigpipe_dfl()?;
+                        #[expect(clippy::disallowed_methods)]
+                        let api = serde_json::to_string_pretty(&*syd::api::API_SPEC).expect("JSON");
+                        stdout().write_all(api.as_bytes())?;
                         return Ok(ExitCode::SUCCESS);
                     }
                     "-q" => is_quick = true,
@@ -175,7 +189,7 @@ syd::main! {
     // - https://github.com/opencontainers/runc/commit/0a8e4117e7f715d5fbeef398405813ce8e88558b
     // - https://github.com/lxc/lxc/commit/6400238d08cdf1ca20d49bafb85f4e224348bf9d
     // Note: syd's procfs protections is another layer of defense against this.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let cookie = if !is_quick {
         match env::var(ENV_RAND) {
             Ok(cookie0) => {
@@ -229,7 +243,7 @@ syd::main! {
     // Generate unique sandbox id from AT_RANDOM bytes.
     // Allow the user to override by setting SYD_ID.
     // Panic if SYD_ID is incorrectly formatted.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     if let Some(sandbox_id) = env::var_os(ENV_ID) {
         assert_eq!(sandbox_id.len(), 128,
             "PANIC: Sandbox ID in SYD_ID environment variable isn't in correct format!");
@@ -258,7 +272,7 @@ syd::main! {
             .mode(0o400)
             .write(true)
             .create_new(true);
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let mut pid_file = match openopts.open(&pid_fn).map(BufWriter::new) {
             Ok(pid_file) => pid_file,
             Err(error) => {
@@ -296,7 +310,7 @@ syd::main! {
     let mut cmd_envp: SydHashSet<OsString> = SydHashSet::default();
 
     // SYD_PROXY_{HOST,PORT,UNIX} -> proxy/ext/{host,port,unix}
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     match env::var(ENV_PROXY_HOST) {
         Ok(host) => sandbox
             .config(&format!("proxy/ext/host:{host}"))
@@ -304,7 +318,7 @@ syd::main! {
         Err(env::VarError::NotPresent) => {}
         Err(error) => panic!("Invalid UTF-8 in {ENV_PROXY_HOST}: {error}"),
     };
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     match env::var(ENV_PROXY_PORT) {
         Ok(port) => sandbox
             .config(&format!("proxy/ext/port:{port}"))
@@ -312,7 +326,7 @@ syd::main! {
         Err(env::VarError::NotPresent) => {}
         Err(error) => panic!("Invalid UTF-8 in {ENV_PROXY_PORT}: {error}"),
     };
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     match env::var(ENV_PROXY_UNIX) {
         Ok(unix) => sandbox
             .config(&format!("proxy/ext/unix:{unix}"))
@@ -363,19 +377,19 @@ syd::main! {
             Short('v') | Long("verbose") => sandbox.verbose = true,
             Long("el") => {
                 set_sigpipe_dfl()?;
-                printf!("{SYD_EL}")?;
+                stdout().write_all(SYD_EL.as_bytes())?;
                 return Ok(ExitCode::SUCCESS);
             }
             Long("sh") => {
                 set_sigpipe_dfl()?;
-                printf!("{ESYD_SH}")?;
+                stdout().write_all(ESYD_SH.as_bytes())?;
                 return Ok(ExitCode::SUCCESS);
             }
             Long("api") => {
                 set_sigpipe_dfl()?;
-                #[allow(clippy::disallowed_methods)]
+                #[expect(clippy::disallowed_methods)]
                 let api = serde_json::to_string_pretty(&*syd::api::API_SPEC).expect("JSON");
-                printf!("{api}")?;
+                stdout().write_all(api.as_bytes())?;
                 return Ok(ExitCode::SUCCESS);
             }
             Short('q') => {} // Ignore, must be first!
@@ -516,7 +530,7 @@ syd::main! {
         (true, _, _) => OsString::from(OsStr::from_bytes(MAGIC_FILE)),
         (false, Some(argv0), false) => argv0,
         (false, None, false) | (false, _, true) => {
-            #[allow(clippy::disallowed_methods)]
+            #[expect(clippy::disallowed_methods)]
             if cmd_arg0.is_none() {
                 // Allow user to override with -a.
                 cmd_arg0 = Some(match Path::new(&sh).file_name() {
@@ -563,63 +577,68 @@ syd::main! {
         sandbox.set_unshare_pid(true);
     }
 
-    let pty_debug = env::var_os("SYD_PTY_DEBUG").is_some();
     let pty_child = if sandbox.has_pty()
         && isatty(std::io::stdin()).unwrap_or(false)
         && isatty(std::io::stdout()).unwrap_or(false)
     {
-        // Step 1: Create a PIDFd of this process and clear O_CLOEXEC.
+        // Create a PIDFd of this process and clear O_CLOEXEC.
         let pidfd = pidfd_open(getpid(), libc::PIDFD_NONBLOCK)?;
         set_cloexec(&pidfd, false)?;
 
-        // Step 2: Open a pseudoterminal device without O_CLOEXEC.
-        let pty_m = posix_openpt(OFlag::O_RDWR | OFlag::O_NOCTTY | OFlag::O_NOFOLLOW)?;
+        // Open a pseudoterminal device and clear O_CLOEXEC.
+        let pty_m = openpt(OFlag::O_RDWR | OFlag::O_NOCTTY)?;
+        set_cloexec(&pty_m, false)?;
+
+        // Grant access to PTY and unlock.
         grantpt(&pty_m)?;
         unlockpt(&pty_m)?;
-        let pty_s = ptsname_r(&pty_m).map(XPathBuf::from)?;
-        #[allow(clippy::disallowed_methods)]
-        let pty_s = open(
-            &pty_s,
-            OFlag::O_RDWR | OFlag::O_NOCTTY | OFlag::O_NOFOLLOW | OFlag::O_CLOEXEC,
-            Mode::empty(),
-        )?;
 
-        // Step 3: Prepare environment of the syd-pty process.
+        // Open peer device.
+        // We are going to pass this end to the sandbox process.
+        // This uses TIOCGPTPEER ioctl(2) so O_NOFOLLOW is not needed.
+        let pty_s = openpts(&pty_m, OFlag::O_RDWR | OFlag::O_NOCTTY | OFlag::O_CLOEXEC)?;
+
+        // Prepare environment of the syd-pty process.
         // Filter the environment variables to only include the list below:
         // 1. PATH
         // 2. LD_LIBRARY_PATH
-        // 3. SYD_PTY_DEBUG
-        // 4. SYD_PTY_RULES
-        let safe_env: SydHashMap<_, _> = env::vars_os()
-            .filter(|(key, _)| {
-                matches!(
-                    key.as_bytes(),
-                    b"PATH" | b"LD_LIBRARY_PATH" | b"SYD_PTY_DEBUG" | b"SYD_PTY_RULES"
-                )
-            })
-            .collect();
+        // 3. SYD_PTY_RULES
+        // We do not need to pass SYD_PTY_DEBUG because we use -d as needed.
+        let safe_env: &[&[u8]] = &[b"PATH", b"LD_LIBRARY_PATH", b"SYD_PTY_RULES"];
 
-        // Step 5: Spawn syd-pty process, and pass PTY main end to it.
-        let mut cmd = std::process::Command::new(
-            env::var_os("CARGO_BIN_EXE_syd-pty").unwrap_or(OsString::from("syd-pty")),
+        // Spawn syd-pty process, and pass PTY main end to it.
+        let pty_debug = env::var_os("SYD_PTY_DEBUG").is_some();
+        let mut cmd = if let Some(cmd) = env::var_os("CARGO_BIN_EXE_syd-pty") {
+            std::process::Command::new(cmd)
+        } else {
+            std::process::Command::new("syd-pty")
+        };
+        cmd.stdin(Stdio::inherit());
+        cmd.stdout(Stdio::inherit());
+        cmd.env_clear();
+        cmd.envs(env::vars_os()
+            .filter(|(key, _)| {
+                    safe_env.iter().any(|env| is_equal(key.as_bytes(), env))
+            })
         );
         if pty_debug {
             cmd.arg("-d");
+            cmd.stderr(Stdio::inherit());
+        } else {
+            cmd.stderr(Stdio::null());
         }
-        cmd.arg(format!("-p{}", pidfd.as_raw_fd()));
-        cmd.arg(format!("-i{}", pty_m.as_raw_fd()));
+        let mut buf = itoa::Buffer::new();
+        cmd.arg("-p");
+        cmd.arg(buf.format(pidfd.as_raw_fd()));
+        cmd.arg("-i");
+        cmd.arg(buf.format(pty_m.as_raw_fd()));
         if let Some(ws) = sandbox.pty_ws_x() {
-            cmd.arg(format!("-x{ws}"));
+            cmd.arg("-x");
+            cmd.arg(buf.format(ws));
         }
         if let Some(ws) = sandbox.pty_ws_y() {
-            cmd.arg(format!("-y{ws}"));
-        }
-        cmd.env_clear();
-        cmd.envs(safe_env);
-        cmd.stdin(Stdio::inherit());
-        cmd.stdout(Stdio::inherit());
-        if !pty_debug {
-            cmd.stderr(Stdio::null());
+            cmd.arg("-y");
+            cmd.arg(buf.format(ws));
         }
         if let Err(error) = cmd.spawn() {
             let errno = err2no(&error);
@@ -639,9 +658,8 @@ syd::main! {
                 "msg": "syd-pty is now forwarding terminal I/O");
         }
 
-        // Step 6: Pass the other end of the PTY pair
-        // to the sandbox process.
-        env::set_var(ENV_PTY_FD, pty_s.as_raw_fd().to_string());
+        // Pass the other end of the PTY pair to the sandbox process.
+        env::set_var(ENV_PTY_FD, buf.format(pty_s.as_raw_fd()));
         Some(pty_s)
     } else {
         env::remove_var(ENV_PTY_FD);
@@ -653,60 +671,60 @@ syd::main! {
         // sandbox/proxy:on implies unshare/net:1
         sandbox.set_unshare_net(true);
 
-        // Step 1: Create a PIDFd of this process.
+        // Create a PIDFd of this process.
         let pidfd = pidfd_open(getpid(), libc::PIDFD_NONBLOCK)?;
 
-        // Step 2: Create a UNIX socket pair.
+        // Create a UNIX socket pair.
         let (stream_parent, stream_child) = UnixStream::pair()?;
 
-        // Step 3: Unset the CLOEXEC flags on the file descriptors.
+        // Unset the CLOEXEC flags on the file descriptors.
         // PIDFds and Rust sockets are automatically CLOEXEC.
         set_cloexec(&pidfd, false)?;
         set_cloexec(&stream_parent, false)?;
 
-        // Step 4: Prepare environment of the syd-tor process.
+        // Prepare environment of the syd-tor process.
         // Filter the environment variables to only include the list below:
         // 1. PATH
         // 2. LD_LIBRARY_PATH
-        // 3. SYD_TOR_DEBUG
-        // 4. SYD_TOR_RULES
-        let safe_env: SydHashMap<_, _> = env::vars_os()
-            .filter(|(key, _)| {
-                matches!(
-                    key.as_bytes(),
-                    b"PATH" | b"LD_LIBRARY_PATH" | b"SYD_TOR_DEBUG" | b"SYD_TOR_RULES"
-                )
-            })
-            .collect();
+        // 3. SYD_TOR_RULES
+        // We do not need to pass SYD_TOR_DEBUG because we use -d as needed.
+        let safe_env: &[&[u8]] = &[b"PATH", b"LD_LIBRARY_PATH", b"SYD_TOR_RULES"];
 
-        // Step 5: Spawn syd-tor process outside the namespace.
+        // Spawn syd-tor process outside the namespace.
         // Pass one end of the socket-pair to it.
-        let mut cmd = std::process::Command::new(
-            env::var_os("CARGO_BIN_EXE_syd-tor").unwrap_or(OsString::from("syd-tor")),
+        let mut cmd = if let Some(cmd) = env::var_os("CARGO_BIN_EXE_syd-tor") {
+            std::process::Command::new(cmd)
+        } else {
+            std::process::Command::new("syd-tor")
+        };
+        cmd.process_group(0);
+        cmd.stdin(Stdio::inherit());
+        cmd.stdout(Stdio::inherit());
+        cmd.env_clear();
+        cmd.envs(env::vars_os()
+            .filter(|(key, _)| {
+                    safe_env.iter().any(|env| is_equal(key.as_bytes(), env))
+            })
         );
         if proxy_debug {
             cmd.arg("-d");
+            cmd.stderr(Stdio::inherit());
+        } else {
+            cmd.stderr(Stdio::null());
         }
-        cmd.arg(format!("-p{}", pidfd.as_raw_fd()));
-        cmd.arg(format!("-i{}", stream_parent.as_raw_fd()));
+        let mut buf = itoa::Buffer::new();
+        cmd.arg("-p");
+        cmd.arg(buf.format(pidfd.as_raw_fd()));
+        cmd.arg("-i");
+        cmd.arg(buf.format(stream_parent.as_raw_fd()));
         // proxy/ext/unix has precedence over proxy/ext/host.
         if let Some(ref proxy_ext_unix) = sandbox.proxy_ext_unix {
             cmd.arg("-u");
             cmd.arg(proxy_ext_unix);
         } else {
+            let addr = format!("{}:{}", sandbox.proxy_ext_addr, sandbox.proxy_ext_port);
             cmd.arg("-o");
-            cmd.arg(format!(
-                "{}:{}",
-                sandbox.proxy_ext_addr, sandbox.proxy_ext_port
-            ));
-        }
-        cmd.env_clear();
-        cmd.envs(safe_env);
-        cmd.process_group(0);
-        cmd.stdin(Stdio::null());
-        cmd.stdout(Stdio::null());
-        if !proxy_debug {
-            cmd.stderr(Stdio::null());
+            cmd.arg(addr);
         }
         if let Err(error) = cmd.spawn() {
             let errno = err2no(&error);
@@ -717,22 +735,15 @@ syd::main! {
             return Err(error.into());
         }
         drop(pidfd);
-
-        let proxy_repr = sandbox
-            .proxy_ext_unix
-            .as_ref()
-            .map(|proxy_ext_unix| proxy_ext_unix.to_string())
-            .unwrap_or_else(|| format!("{}!{}", sandbox.proxy_ext_addr, sandbox.proxy_ext_port));
         if proxy_debug {
             warn!("ctx": "spawn_proxy",
-                "msg": format!("proxy is now forwarding external traffic to {proxy_repr}"));
+                "msg": format!("proxy is now forwarding external traffic to {}", sandbox.proxy_repr()));
         } else {
             info!("ctx": "spawn_proxy",
-                "msg": format!("proxy is now forwarding external traffic to {proxy_repr}"));
+                "msg": format!("proxy is now forwarding external traffic to {}", sandbox.proxy_repr()));
         }
 
-        // Step 4: Pass the other end of the socket-pair
-        // to the new namespace.
+        // Pass the other end of the socket-pair to the new namespace.
         Some((stream_child, sandbox.proxy_port))
     } else {
         None
@@ -769,42 +780,14 @@ syd::main! {
         let gid = getgid().as_raw();
 
         let map_root = sandbox.flags.map_root();
-
         let uid_buf = {
-            let uid_maps = vec![
-                UidMap {
-                    inside_uid: if map_root { 0 } else { uid },
-                    outside_uid: uid,
-                    count: 1,
-                }, // Map the current user.
-            ];
             let mut buf = Vec::new();
-            for map in uid_maps {
-                writeln!(
-                    &mut buf,
-                    "{} {} {}",
-                    map.inside_uid, map.outside_uid, map.count
-                )?;
-            }
+            write!(&mut buf, "{} {uid} 1", if map_root { 0 } else { uid })?;
             buf
         };
-
         let gid_buf = {
-            let gid_maps = vec![
-                GidMap {
-                    inside_gid: if map_root { 0 } else { gid },
-                    outside_gid: gid,
-                    count: 1,
-                }, // Map the current group.
-            ];
             let mut buf = Vec::new();
-            for map in gid_maps {
-                writeln!(
-                    &mut buf,
-                    "{} {} {}",
-                    map.inside_gid, map.outside_gid, map.count
-                )?;
-            }
+            write!(&mut buf, "{} {gid} 1", if map_root { 0 } else { gid })?;
             buf
         };
         Some((uid_buf, gid_buf))
@@ -838,12 +821,12 @@ syd::main! {
     }
 
     // Write uid/gid map for user namespace.
-    #[allow(clippy::disallowed_methods)]
-    if let Some((ref uid_buf, ref gid_buf)) = id_buf {
+    let pid = Pid::this();
+    if let Some((uid_buf, gid_buf)) = id_buf {
         // Write "deny" to /proc/self/setgroups before writing to gid_map.
-        File::create("/proc/self/setgroups").and_then(|mut f| f.write_all(b"deny"))?;
-        File::create("/proc/self/gid_map").and_then(|mut f| f.write_all(&gid_buf[..]))?;
-        File::create("/proc/self/uid_map").and_then(|mut f| f.write_all(&uid_buf[..]))?;
+        proc_write(pid, b"setgroups", b"deny")?;
+        proc_write(pid, b"gid_map", &gid_buf)?;
+        proc_write(pid, b"uid_map", &uid_buf)?;
 
         // Set inheritable mask and ambient caps to retain caps after execve(2).
         caps::securebits::set_keepcaps(true)?;
@@ -968,26 +951,34 @@ syd::main! {
     }
 
     if sandbox.flags.unshare_time() {
-        #[allow(clippy::disallowed_methods)]
-        if let Some(offset) = sandbox.time {
-            let mut file = File::create("/proc/self/timens_offsets")?;
-            let contents = format!("monotonic {offset} 0\nboottime {offset} 0\n");
-            file.write_all(contents.as_bytes())?;
-            info!("ctx": "set_boot_time", "msg": "set boot time in time namespace");
-        } else if let Ok(boottime) = clock_gettime(ClockId::CLOCK_BOOTTIME) {
-            // Set uptime to 0 for time namespace. Ignore errors.
-            if let Ok(mut file) = File::create("/proc/self/timens_offsets") {
-                #[allow(clippy::arithmetic_side_effects)]
-                let contents = format!("boottime {} 0\n", -boottime.tv_sec());
-                let _ = file.write_all(contents.as_bytes());
-            }
-            info!("ctx": "set_boot_time", "msg": "reset boot time in time namespace");
-        }
+        let (moffset, boffset) = if let Some(offset) = sandbox.time {
+            (offset, offset)
+        } else {
+            let mtime = clock_gettime(ClockId::CLOCK_MONOTONIC)?;
+            let btime = clock_gettime(ClockId::CLOCK_BOOTTIME)?;
+            let moffset = mtime.tv_sec().checked_neg().ok_or(Errno::EOVERFLOW)?;
+            let boffset = btime.tv_sec().checked_neg().ok_or(Errno::EOVERFLOW)?;
+            // Into is necessary on 32-bit.
+            #[expect(clippy::useless_conversion)]
+            (moffset.into(), boffset.into())
+        };
+
+        // <clock-id> <offset-secs> <offset-nanosecs>
+        let mut buf = itoa::Buffer::new();
+        let mut data = "monotonic ".to_string();
+        data.push_str(buf.format(moffset));
+        data.push_str(" 0\nboottime ");
+        data.push_str(buf.format(boffset));
+        data.push_str(" 0\n");
+
+        proc_write(pid, b"timens_offsets", data.as_bytes())?;
+        info!("ctx": "set_boot_time", "msg": "set boot time in time namespace",
+            "timens_offsets": [moffset, boffset]);
     }
 
     // Disable Speculative Store Bypass mitigations
-    // for trace/allow_unsafe_spec_exec:1
-    let ssb = sandbox.flags.allow_unsafe_spec_exec();
+    // for trace/allow_unsafe_exec_speculative:1
+    let ssb = sandbox.flags.allow_unsafe_exec_speculative();
 
     // SAFETY: fork is our safest option here.
     let child = match unsafe { fork() }? {
@@ -1051,7 +1042,7 @@ syd::main! {
     ctx.set_ctl_nnp(true)?;
 
     // Disable Speculative Store Bypass mitigations
-    // with trace/allow_unsafe_spec_exec:1
+    // with trace/allow_unsafe_exec_speculative:1
     ctx.set_ctl_ssb(ssb)?;
 
     // DO NOT synchronize filter to all threads.
@@ -1074,7 +1065,7 @@ syd::main! {
         "exit_group",
         "waitid",
         "brk",
-        "madvise",
+        //"madvise", advice are confined.
         "mremap",
         "munmap",
         "sigaction",
@@ -1101,6 +1092,9 @@ syd::main! {
         }
     }
 
+    // Allow safe madvise(2) advice.
+    confine_scmp_madvise(&mut ctx)?;
+
     // Allow write(2) to standard error.
     if let Ok(syscall) = ScmpSyscall::from_name("write") {
         ctx.add_rule_conditional(
@@ -1113,7 +1107,7 @@ syd::main! {
     // Prevent executable memory.
     const PROT_EXEC: u64 = libc::PROT_EXEC as u64;
     for name in ["mmap", "mmap2", "mprotect"] {
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         ctx.add_rule_conditional(
             ScmpAction::Allow,
             ScmpSyscall::from_name(name).unwrap(),
@@ -1124,13 +1118,13 @@ syd::main! {
     ctx.load()?;
 
     loop {
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_sign_loss)]
         break match waitid(Id::Pid(child), WaitPidFlag::WEXITED) {
             Ok(WaitStatus::Exited(_, code)) =>
             {
-                #[allow(clippy::cast_possible_truncation)]
-                #[allow(clippy::cast_sign_loss)]
+                #[expect(clippy::cast_possible_truncation)]
+                #[expect(clippy::cast_sign_loss)]
                 Ok(ExitCode::from(code as u8))
             }
             Ok(WaitStatus::Signaled(_, signal, _)) => {
