@@ -69,7 +69,7 @@ pub(crate) enum SysInfo {
 
 #[inline]
 fn fill_pod_random<T>(pod: &mut T) -> Result<(), Errno> {
-    let siz = std::mem::size_of::<T>();
+    let siz = size_of::<T>();
     let ptr = (pod as *mut T) as *mut u8;
     // SAFETY: ptr is a valid sysinfo32 or sysinfo64.
     let bytes = unsafe { std::slice::from_raw_parts_mut(ptr, siz) };
@@ -213,17 +213,11 @@ impl SysInfo {
         match self {
             // SAFETY: s is a valid sysinfo32 and we only read its memory as bytes.
             Self::S32(s) => unsafe {
-                std::slice::from_raw_parts(
-                    (s as *const _) as *const u8,
-                    std::mem::size_of::<sysinfo32>(),
-                )
+                std::slice::from_raw_parts((s as *const _) as *const u8, size_of::<sysinfo32>())
             },
             // SAFETY: s is a valid sysinfo64 and we only read its memory as bytes.
             Self::S64(s) => unsafe {
-                std::slice::from_raw_parts(
-                    (s as *const _) as *const u8,
-                    std::mem::size_of::<sysinfo64>(),
-                )
+                std::slice::from_raw_parts((s as *const _) as *const u8, size_of::<sysinfo64>())
             },
         }
     }
@@ -285,7 +279,7 @@ impl RandTimer {
         let repr = self.proc();
         let data = repr.as_bytes();
 
-        let fd = safe_memfd_create(c"syd-proc-uptime", *SAFE_MFD_FLAGS)?;
+        let fd = safe_memfd_create(c"syd/proc/uptime", *SAFE_MFD_FLAGS)?;
         let mut file = File::from(fd);
 
         file.write_all(data).or(Err(Errno::EIO))?;
