@@ -24,6 +24,7 @@ use std::{
     time::Duration,
 };
 
+use memchr::memmem;
 use nix::{
     errno::Errno,
     sys::{
@@ -245,6 +246,13 @@ macro_rules! assert_status_hidden {
     };
 }
 
+#[macro_export]
+macro_rules! assert_status_notdir {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::ENOTDIR);
+    };
+}
+
 #[macro_export]
 macro_rules! assert_status_invalid {
     ($status:expr) => {
@@ -326,6 +334,24 @@ macro_rules! assert_status_operation_not_supported {
     };
 }
 
+#[macro_export]
+macro_rules! skip_unless_gnu {
+    ($($program:expr),* $(,)?) => {{
+        let mut all_gnu = true;
+        $(
+            if !is_program_gnu($program) {
+                eprintln!("Test requires {} to be GNU, skipping!", $program);
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                all_gnu = false;
+            }
+        )*
+
+        if !all_gnu {
+            return Ok(());
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_unless_available {
     ($($program:expr),* $(,)?) => {{
@@ -344,6 +370,17 @@ macro_rules! skip_unless_available {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_if_32bin {
+    () => {{
+        if cfg!(target_pointer_width = "32") {
+            eprintln!("32-bit target detected, skipping!");
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_if_32bin_64host {
     () => {{
@@ -529,7 +566,7 @@ macro_rules! skip_unless_xattrs_are_supported {
             Err(errno) => return Err(TestError(format!("Failed to open xattr.test: {errno}"))),
         };
 
-        match syd::fs::fsetxattr(&fd, "user.syd.crypt.api", b"3", libc::XATTR_CREATE) {
+        match syd::xattr::fsetxattr(&fd, "user.syd.crypt.api", b"3", libc::XATTR_CREATE) {
             Ok(_) => {
                 let _ = nix::unistd::close(fd);
             }
@@ -1173,6 +1210,22 @@ pub fn syd() -> Syd {
     cmd
 }
 
+/// Check if the given program is of GNU flavour.
+pub fn is_program_gnu(command: &str) -> bool {
+    if check_32bin_64host() {
+        eprintln!("Binary/Host mismatch, cannot use program {command}!");
+        return false;
+    }
+
+    // Check if `command --version` returns GNU substring.
+    let out = Command::new(command)
+        .arg("--version")
+        .output()
+        .unwrap()
+        .stdout;
+    memmem::find(&out, b"GNU").is_some()
+}
+
 /// 0. Check if there's a binary/host mismatch.
 /// 1. Check if a program exists in PATH
 pub fn is_program_available(command: &str) -> bool {
