@@ -277,7 +277,7 @@ impl Command {
         // Read the value of the file descriptor from the pipe.
         // Handle interrupts and partial reads.
         // EOF means process died before writing to the pipe.
-        let mut buf = vec![0u8; std::mem::size_of::<RawFd>()];
+        let mut buf = vec![0u8; size_of::<RawFd>()];
         let mut nread = 0;
         while nread < buf.len() {
             #[expect(clippy::arithmetic_side_effects)]
