@@ -18,6 +18,16 @@ use nix::{
 };
 use serde_json::json;
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
@@ -84,7 +94,7 @@ syd::main! {
         .to_string_lossy()
         .into_owned();
 
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let aux = json!({
         "uid": at_uid,
         "euid": at_euid,
@@ -103,7 +113,7 @@ syd::main! {
         "platform": at_platform,
     });
 
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let aux = serde_json::to_string_pretty(&aux).unwrap();
     println!("{aux}");
 
