@@ -162,6 +162,7 @@ fn list() {
     println!("lib");
     println!("linux");
     println!("ltp");
+    println!("nix");
     println!("noipv4");
     println!("noipv6");
     println!("nomem");
@@ -249,6 +250,15 @@ fn dump(name: &str) -> SydResult<()> {
                 println!("{command}");
             }
         }
+        "nix" => {
+            println!("# Syd profile: NIX");
+            println!("# Number of rules: {}", syd::config::PROFILE_NIX.len());
+            println!("# Copyright (c) 2025 Ali Polatel <alip@chesswob.org>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_NIX {
+                println!("{command}");
+            }
+        }
         "tty" => {
             println!("# Syd profile: TTY");
             println!("# Number of rules: {}", syd::config::PROFILE_TTY.len());
