@@ -19,14 +19,16 @@ use std::{
 use memchr::memchr;
 use nix::{
     errno::Errno,
-    fcntl::{open, readlinkat, OFlag},
+    fcntl::{open, OFlag},
     sys::stat::Mode,
     unistd::{dup2_raw, getpid, Pid},
 };
 use syd::{
     compat::getdents64,
     config::*,
-    fs::{fd_status_flags, parse_fd, pidfd_getfd, pidfd_open, set_cloexec, PIDFD_THREAD},
+    fs::{
+        fd_status_flags, parse_fd, pidfd_getfd, pidfd_open, readlinkat, set_cloexec, PIDFD_THREAD,
+    },
     path::{XPath, XPathBuf},
     rng::duprand,
 };
@@ -234,8 +236,9 @@ fn proc_pid_fd(pid: Option<Pid>) -> Result<Vec<(RawFd, XPathBuf)>, Errno> {
         Mode::empty(),
     )?;
 
-    let mut dot = 0u8;
     let mut res = vec![];
+    let mut seen_dot = false;
+    let mut seen_dotdot = false;
     loop {
         let mut entries = match getdents64(&dir, DIRENT_BUF_SIZE) {
             Ok(entries) => entries,
@@ -244,12 +247,17 @@ fn proc_pid_fd(pid: Option<Pid>) -> Result<Vec<(RawFd, XPathBuf)>, Errno> {
         };
 
         for entry in &mut entries {
-            if dot < 2 && entry.is_dot() {
-                dot += 1;
+            if !seen_dot && entry.is_dot() {
+                seen_dot = true;
                 continue;
             }
-            let fd = parse_fd(XPath::from_bytes(entry.name_bytes()))?;
-            let target = readlinkat(&dir, entry.name_bytes()).map(XPathBuf::from)?;
+            if !seen_dotdot && entry.is_dotdot() {
+                seen_dotdot = true;
+                continue;
+            }
+            let entry = XPath::from_bytes(entry.name_bytes());
+            let fd = parse_fd(entry)?;
+            let target = readlinkat(&dir, entry)?;
 
             res.push((fd, target));
         }
