@@ -17,6 +17,16 @@ use data_encoding::{HEXLOWER, HEXLOWER_PERMISSIVE};
 use memchr::arch::all::is_equal;
 use nix::{errno::Errno, unistd::isatty};
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
@@ -74,7 +84,7 @@ syd::main! {
     }
 
     // Lock stdin for efficient reading
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let mut input: Box<dyn BufRead> = match opt_input {
         None => Box::new(std::io::stdin().lock()),
         Some(path) if is_equal(path.as_bytes(), b"-") => Box::new(std::io::stdin().lock()),
