@@ -11,6 +11,16 @@ use std::{mem::MaybeUninit, process::ExitCode};
 use nix::errno::Errno;
 use serde_json::json;
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
@@ -18,7 +28,7 @@ syd::main! {
 
     // Parse CLI options.
     let mut parser = lexopt::Parser::from_env();
-    #[allow(clippy::never_loop)]
+    #[expect(clippy::never_loop)]
     while let Some(arg) = parser.next()? {
         match arg {
             Short('h') => {
@@ -35,7 +45,7 @@ syd::main! {
     // SAFETY: sysinfo() has initialized `info` if it succeeded.
     let info = unsafe { info.assume_init() };
 
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let info = json!({
         "uptime": info.uptime,
         "loads": info.loads,
@@ -51,7 +61,7 @@ syd::main! {
         "mem_unit": info.mem_unit,
     });
 
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let info = serde_json::to_string_pretty(&info).unwrap();
     println!("{info}");
 
