@@ -22,7 +22,7 @@ use nix::{
     unistd::Pid,
 };
 use syd::{
-    fs::{safe_canonicalize, FsFlags},
+    lookup::{safe_canonicalize, FsFlags},
     path::XPathBuf,
     sandbox::{Flags, Sandbox},
     syslog::LogLevel,
