@@ -26,6 +26,16 @@ use syd::{
     proc::proc_namespaces,
 };
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
