@@ -9,8 +9,8 @@
 use std::{
     env,
     fs::File,
-    io::{Read, Seek, SeekFrom, Write},
-    os::fd::AsRawFd,
+    io::{Read, Seek, SeekFrom, Stdin, Write},
+    os::fd::{AsFd, AsRawFd, BorrowedFd},
     process::{exit, ExitCode},
     time::Duration,
 };
@@ -25,8 +25,10 @@ use syd::{
     config::*,
     cookie::safe_memfd_create,
     debug,
-    fs::{safe_copy_if_exists, seal_memfd, set_cloexec},
+    fs::{seal_memfd, set_cloexec},
     get_user_home, get_user_name, human_size,
+    io::ReadFd,
+    lookup::safe_copy_if_exists,
     path::XPathBuf,
     syslog::LogLevel,
     wordexp::*,
@@ -42,6 +44,31 @@ static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
 #[global_allocator]
 static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
 
+enum Input {
+    File(File),
+    Stdin(Stdin),
+}
+
+impl Read for Input {
+    fn read(&mut self, buf: &mut [u8]) -> std::io::Result<usize> {
+        match self {
+            Input::File(f) => f.read(buf),
+            Input::Stdin(s) => s.read(buf),
+        }
+    }
+}
+
+impl AsFd for Input {
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        match self {
+            Input::File(f) => f.as_fd(),
+            Input::Stdin(s) => s.as_fd(),
+        }
+    }
+}
+
+impl ReadFd for Input {}
+
 syd::main! {
     use lexopt::prelude::*;
 
@@ -199,22 +226,22 @@ syd::main! {
 
     #[expect(clippy::type_complexity)]
     #[expect(clippy::disallowed_methods)]
-    let input: Option<(Box<dyn Read>, String)> = if let Some(path) = args.next() {
+    let input: Option<(Input, String)> = if let Some(path) = args.next() {
         Some((
-            Box::new(File::open(&path)?),
+            Input::File(File::open(&path)?),
             XPathBuf::from(path).to_string(),
         ))
     } else if isatty(std::io::stdin()).unwrap_or(false) {
         None
     } else {
-        Some((Box::new(std::io::stdin()), "standard input".to_string()))
+        Some((Input::Stdin(std::io::stdin()), "standard input".to_string()))
     };
 
     if let Some((mut input_file, input_name)) = input {
         debug!("ctx": "sh",
             "msg": format!("copying from {input_name} to memory-file {}...",
                 file.as_raw_fd()));
-        let copylen = std::io::copy(&mut input_file, &mut file)?;
+        let copylen = syd::io::copy(&mut input_file, &mut file)?;
         debug!("ctx": "sh",
             "msg": format!("copied {} from {input_name} to memory-file {}",
                 human_size(copylen.try_into()?),
@@ -302,7 +329,7 @@ syd::main! {
 
         // rewrite history!
         file.seek(SeekFrom::Start(0))?;
-        let copylen = std::io::copy(&mut file, &mut fdup)?;
+        let copylen = syd::io::copy(&mut file, &mut fdup)?;
         debug!("ctx": "sh",
             "msg": format!("copied {} from memory-file {} to {}",
                 human_size(copylen.try_into()?),
