@@ -13,7 +13,7 @@ use std::{
 
 use nix::{libc::pid_t, unistd::Pid};
 use serde_json::json;
-use syd::proc::{proc_stat, proc_status};
+use syd::proc::{proc_cmdline, proc_comm, proc_stat, proc_status};
 
 // Set global allocator to mimalloc.
 #[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
@@ -32,23 +32,27 @@ syd::main! {
     syd::config::proc_init_simple()?;
 
     let pid = match std::env::args().nth(1).map(|arg| arg.parse::<pid_t>()) {
-        Some(Ok(pid)) => pid,
-        None => Pid::this().as_raw(),
+        Some(Ok(pid)) => Pid::from_raw(pid),
+        None => Pid::this(),
         Some(Err(_)) => {
             help();
             return Ok(ExitCode::SUCCESS);
         }
     };
 
-    let stat = proc_stat(Pid::from_raw(pid))?;
-    let status = proc_status(Pid::from_raw(pid))?;
+    let comm = proc_comm(pid)?;
+    let cmdline = proc_cmdline(pid)?;
+    let stat = proc_stat(pid)?;
+    let status = proc_status(pid)?;
     #[expect(clippy::disallowed_methods)]
     let status = json!({
-        "pid": pid,
-        "tgid": status.pid,
-        "command": status.command,
-        "umask": status.umask,
+        "pid": pid.as_raw(),
+        "tgid": status.pid.as_raw(),
+        "comm": comm,
+        "cmdline": cmdline,
+        "umask": status.umask.bits(),
         "tty_nr": stat.tty_nr,
+        "start_brk": stat.startbrk,
         "startstack": stat.startstack,
         "num_threads": stat.num_threads,
         "sig": {
