@@ -47,12 +47,13 @@ use crate::{
     debug,
     err::{err2no, SydJoinHandle, SydResult},
     error,
-    fs::{seal_memfd, FileInfo},
+    fs::seal_memfd,
     hash::{
         aes_ctr_enc, aes_ctr_init, hmac_sha256_feed, hmac_sha256_fini, hmac_sha256_init,
         SydHashMap, BLOCK_SIZE, HMAC_TAG_SIZE, IV, IV_SIZE, SYD3_HDR_SIZE,
     },
     info,
+    lookup::FileInfo,
     ofd::lock_fd,
     path::{XPath, XPathBuf},
     retry::{retry_on_eintr, retry_on_intr},
