@@ -20,7 +20,7 @@ use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext, ScmpSyscall};
 use nix::{
     errno::Errno,
     sched::{unshare, CloneFlags},
-    unistd::{getpid, write, Gid, Pid, Uid},
+    unistd::{getpid, lseek64, write, Gid, Pid, Uid, Whence},
 };
 use serde::{ser::SerializeMap, Serialize, Serializer};
 
@@ -34,7 +34,7 @@ use crate::{
     error,
     fs::{closeexcept, seccomp_notify_id_valid, tgkill},
     info,
-    proc::proc_status,
+    proc::{proc_status_open, proc_status_read},
     retry::retry_on_eintr,
     sandbox::Flags,
     sigset::SydSigSet,
@@ -227,7 +227,10 @@ impl Interrupter {
                     .unwrap_or_else(|err| err.into_inner());
 
                 // Handle interrupts as necessary.
-                map.retain(|_, interrupt| self.handle_interrupt(tgid, interrupt));
+                map.retain_mut(|interrupt| self.handle_interrupt(tgid, interrupt));
+
+                // Keep memory usage minimal.
+                map.shrink_to_fit();
             }
 
             // Check if it's the time to exit.
@@ -239,30 +242,65 @@ impl Interrupter {
         Ok(())
     }
 
-    fn handle_interrupt(&self, tgid: Pid, interrupt: &SysInterrupt) -> bool {
-        // Check pending signals for the thread.
+    // Handles syscall signal interrupts.
+    //
+    // Returns false if interrupt is handled, true otherwise.
+    fn handle_interrupt(&self, tgid: Pid, interrupt: &mut SysInterrupt) -> bool {
+        // Check if syd_emu is already done with the request.
+        if interrupt.delete {
+            return false;
+        }
+
+        // Open proc_pid_status(5) if it's not open already.
         //
         // SAFETY:
         // 1. We want to wake the respective syd_emu thread in case the
         //    process is no longer valid otherwise we may end up with a
         //    deadlock: See miniupnpc tests, thx kepstin!
         // 2. To prevent PID reuse vectors we validate the request ID.
-        #[expect(clippy::cast_possible_wrap)]
-        let pid = Pid::from_raw(interrupt.request.pid as libc::pid_t);
-        let status = match proc_status(pid) {
-            Ok(status) if self.is_valid(interrupt.request.id) => {
-                // SAFETY: seccomp-id validated, proc_pid_status(5) is valid.
-                status
+        let status_fd = if let Some(fd) = interrupt.status.as_ref() {
+            if let Err(errno) = lseek64(fd, 0, Whence::SeekSet) {
+                if self.is_valid(interrupt.request.id) {
+                    let source = InterruptSource::ProcessInvalid(errno);
+                    Self::interrupt(tgid, interrupt, source);
+                } // no need to interrupt for invalid seccomp-id.
+                return false;
             }
+            fd
+        } else {
+            let fd = match proc_status_open(interrupt.request.pid()) {
+                Ok(fd) if self.is_valid(interrupt.request.id) => {
+                    // seccomp-id validated, proc_pid_status(5) is valid.
+                    fd
+                }
+                Err(errno) if self.is_valid(interrupt.request.id) => {
+                    let source = InterruptSource::ProcessInvalid(errno);
+                    Self::interrupt(tgid, interrupt, source);
+                    return false;
+                }
+                // seccomp-id invalid, no need to interrupt.
+                _ => return false,
+            };
+            interrupt.status = Some(fd);
+            #[expect(clippy::disallowed_methods)]
+            interrupt.status.as_ref().unwrap()
+        };
+
+        // Parse proc_pid_status(5).
+        let status = match proc_status_read(status_fd) {
+            Ok(status) => status,
             Err(errno) if self.is_valid(interrupt.request.id) => {
                 let source = InterruptSource::ProcessInvalid(errno);
                 Self::interrupt(tgid, interrupt, source);
                 return false;
             }
+            // seccomp-id invalid, no need to interrupt.
             _ => return false,
         };
 
-        // Check for per-{thread,process} pending signals:
+        // Check pending signals for the thread.
+        //
+        // 1. Check for per-{thread,process} pending signals:
         //
         // Aggregate pending signals from both the thread-local and
         // process-global queues. In Linux, each thread (LWP) has its
@@ -273,7 +311,7 @@ impl Interrupter {
         // interrupt this execution context.
         let mut sigset = status.sig_pending_thread | status.sig_pending_process;
 
-        // Exclude per-thread blocked signals:
+        // 2. Exclude per-thread blocked signals:
         //
         // Remove those signals currently masked by this thread's signal
         // mask. Signal masks are maintained per-thread via
@@ -288,12 +326,9 @@ impl Interrupter {
             return true;
         }
 
-        // Filter out restarting signals per-process,
-        // unless ignore_restart is set. This may be the
-        // case e.g. when the socket has a timeout for
-        // accept and connect.
-        // Note, `interrupt.ignore_restart` check
-        // was done before calling this function and
+        // 3. Filter out restarting signals per-process, unless ignore_restart is set.
+        // This may be the case e.g. when the socket has a timeout for accept and connect.
+        // Note, `interrupt.ignore_restart` check was done before calling this function and
         // sigset_restart is only Some if it is false.
         if !interrupt.ignore_restart {
             if let Some(sigset_restart) = self
