@@ -35,8 +35,9 @@ use crate::{
     },
     confine::{ScmpNotifReq, SydMemoryMap},
     elf::ExecutableFile,
-    fs::{block_signal, sigtimedpoll, unblock_signal, CanonicalPath},
+    fs::{block_signal, sigtimedpoll, unblock_signal},
     hash::SydHashMap,
+    lookup::CanonicalPath,
     retry::retry_on_eintr,
     sigset::SydSigSet,
 };
@@ -273,8 +274,9 @@ impl<'a> WorkerCache<'a> {
         let (ref lock, ref cvar) = *self.sysint_map.sys_block;
         let mut map = lock.lock().unwrap_or_else(|err| err.into_inner());
 
-        map.retain(|_, interrupt| interrupt.handler != handler_tid);
-        map.insert(request.id, interrupt);
+        map.retain_mut(|interrupt| handler_tid != interrupt.handler || interrupt.delete());
+        map.try_reserve(1).or(Err(Errno::ENOMEM))?;
+        map.push(interrupt);
 
         cvar.notify_one();
 
@@ -295,7 +297,7 @@ impl<'a> WorkerCache<'a> {
         if !interrupted {
             let (ref lock, ref _cvar) = *self.sysint_map.sys_block;
             let mut map = lock.lock().unwrap_or_else(|err| err.into_inner());
-            map.remove(&request_id);
+            map.retain_mut(|interrupt| request_id != interrupt.request.id || interrupt.delete());
         }
 
         Ok(())
